<?xml version="1.0" encoding="UTF-8"?>

<!--
	* Licensed Materials - Property of IBM
	* 5724-C11, 5724-C13, 5724-D17
	* (C) Copyright IBM Corporation 1997, 2003. All Rights Reserved.
	* US Government Users Restricted Rights- Use, duplication or disclosure
	* restricted by GSA ADP Schedule Contract with IBM Corp.
-->

<xsl:stylesheet 
	version="1.0" 
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
	xmlns:fo="http://www.w3.org/1999/XSL/Format" 
	xmlns:bpel="http://schemas.xmlsoap.org/ws/2002/07/business-process/" 
	xmlns:wswf="http://schemas.xmlsoap.org/ws/2002/07/wswf/"
	xmlns:xalan="http://xml.apache.org/xalan"
	xmlns:java="http://xml.apache.org/xslt/java"	
    extension-element-prefixes='xalan'
	exclude-result-prefixes="bpel wswf java fo">
	<xsl:output method="xml" indent="yes"/>
	<xsl:variable name="Version">4.2.0</xsl:variable>
	<xsl:variable name="FormatVersion">4.2.0</xsl:variable>
	<xsl:variable name="ModelElementVersion">3</xsl:variable>
	<!-- 
		Check if the BPEL is generated by CrossWorlds.  The XSLT will preform different transformation if the BPEL
		is created by CrossWorlds based on some special convention.
	-->
	<xsl:variable name="CWBPEL" select="comment()='Generated by IBM CrossWorlds'"/>
	<!-- 
	***************************************************************************************************************************************************************************
	Java extension
	***************************************************************************************************************************************************************************
-->	
	<!-- Create Java Object instance -->
	<!-- UniqueID - obtain unique ID --> 
    <xsl:variable name="uniqueID" select="java:com.ibm.btools.itools.cwconverter.xslt.ext.UniqueID.new()"/>
	<!-- SimpleStrVar - store and retrieve a variable with String type --> 
    <xsl:variable name="simpleStrVar" select="java:com.ibm.btools.itools.cwconverter.xslt.ext.SimpleStrVar.new()"/>
    <!-- NodeSetVar - store and retrieve a variable with NodeSet type -->
    <xsl:variable name="nodeSetVar" select="java:com.ibm.btools.itools.cwconverter.xslt.ext.NodeSetVar.new()"/>	
    
    <!-- Global Variables -->
    <!-- 'Links' is the key to store a link node using NodeSetVar -->
    <xsl:variable name="linksKey" select="'Links'"/>
    <!-- 'SubDiagramRefList' is the key to store the name of subdiagram using SimpleStrVar -->
    <xsl:variable name="subDiagRefList" select="'SubDiagramRefList'"/>

	<!-- 
	***************************************************************************************************************************************************************************
	Fills in the CollabTemplate
	***************************************************************************************************************************************************************************
-->
	<xsl:template match="/">
		<CollabTemplate>
			<xsl:attribute name="Name"><xsl:value-of select="bpel:process/@name"/></xsl:attribute>
			<xsl:attribute name="Version"><xsl:value-of select="$Version"/></xsl:attribute>
			<xsl:attribute name="FormatVersion"><xsl:value-of select="$FormatVersion"/></xsl:attribute>
			<xsl:call-template name="writeHeader"/>
			<!-- create scenarios node and store it in a variable to later-on pass to the writeTemplateDefs
				template.  
			-->
			<xsl:variable name="scenarios">
				<Scenarios>
					<xsl:call-template name="navigateScenarios"/>
				</Scenarios>
			</xsl:variable>
			<xsl:call-template name="writeTemplateDefs">
				<xsl:with-param name="scenarios" select="$scenarios"/>
			</xsl:call-template>
			<GUITemplateDefinition>
				<GUIProperties/>
				<GUIImports/>
				<GUIUserSection1/>
				<GUIUserSection2/>
				<GUIUserSection3/>
				<GUIVariables/>			
			</GUITemplateDefinition>
			<xsl:copy-of select="$scenarios"/>
			<Messages>
				<CommonMessages/>
				<SpecificMessages/>
			</Messages>
		</CollabTemplate>
	</xsl:template>

	<!-- 
	***************************************************************************************************************************************************************************
	Fills in the Collab header
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="writeHeader">
		<Header>
			<Title>CrossWorlds Collaboration Template</Title>
			<Desc>Automatically Transfomed by IBM CrossWorlds Process Designer converter</Desc>
			<CreateTime>
				<xsl:attribute name="Format">ISO8601GMT</xsl:attribute>
			</CreateTime>
			<ModifyTime>
				<xsl:attribute name="Format">ISO8601GMT</xsl:attribute>
			</ModifyTime>
		</Header>
	</xsl:template>
	<!-- 
	***************************************************************************************************************************************************************************
	Fills in the TemplateDefs
	***************************************************************************************************************************************************************************
-->

	<xsl:template name="writeTemplateDefs">
		<xsl:param name="scenarios"/>
		<xsl:variable name="codeFrag">
			<xsl:value-of select="bpel:process/wswf:globals"/>
		</xsl:variable>
		<TemplateDefinition>
			<Desc/>
			<xsl:call-template name="writeProperties">
				<xsl:with-param name="codeFrag" select="$codeFrag"/>
			</xsl:call-template>
			<MinTransactionLevel>
				<xsl:attribute name="Level">None</xsl:attribute>
			</MinTransactionLevel>
			<Package/>
			<LongLivedBusinessProcess Enabled="No"/>	
			<GeneratedVars>
				<xsl:choose>
					<xsl:when test="boolean(normalize-space($codeFrag))">
						<xsl:call-template name="writeGeneratedVarsWithCodeFrag">
							<xsl:with-param name="codeFrag" select="$codeFrag"/>
						</xsl:call-template>
					</xsl:when>
					<xsl:otherwise>
						<xsl:call-template name="writeGeneratedVars"/>
					</xsl:otherwise>
				</xsl:choose>
			</GeneratedVars>
			<BusObjRefs>
				<xsl:call-template name="writeBusObjRefs">
					<xsl:with-param name="scenarios" select="$scenarios"/>
				</xsl:call-template>
			</BusObjRefs>
		</TemplateDefinition>
	</xsl:template>
	<!-- 
	***************************************************************************************************************************************************************************
	Fills in an Attribute
	***************************************************************************************************************************************************************************
-->
   	<xsl:template name="writeAttribute">
		<xsl:param name="attribLines"/>
		<xsl:param name="propType"/>
		<xsl:variable name="attribLine" select="substring-before($attribLines,'&#xA;')"/>
		<xsl:variable name="nextAttribLines" select="substring-after($attribLines,'&#xA;')"/>
		<xsl:if test="boolean(normalize-space($attribLine))">
			<Attribute>
				<xsl:choose>
					<xsl:when test="$propType='PropertyValues'">
						<xsl:attribute name="Name">
							<xsl:value-of select="substring-before($attribLine,'=')"/>
						</xsl:attribute>
						<xsl:attribute name="Type">0</xsl:attribute>
						<Value>
							<xsl:value-of select="substring-after($attribLine,'=')"/>
						</Value>
					</xsl:when>
					<xsl:otherwise>
						<xsl:attribute name="Name"/>
						<xsl:attribute name="Type">0</xsl:attribute>
						<Value>
							<xsl:value-of select="$attribLine"/>
						</Value>
					</xsl:otherwise>
				</xsl:choose>
				<Desc/>
			</Attribute>
		</xsl:if>
		<xsl:if test="boolean(normalize-space($nextAttribLines))">
		   	<xsl:call-template name="writeAttribute">
		   		<xsl:with-param name="attribLines">
		   			<xsl:value-of select="$nextAttribLines"/>
		   		</xsl:with-param>
		   		<xsl:with-param name="propType">
		   			<xsl:value-of select="$propType"/>
		   		</xsl:with-param>
		   	</xsl:call-template>					
		</xsl:if>
	</xsl:template>

<!-- 
	***************************************************************************************************************************************************************************
	Extracts a sub-section of the Code fragment based on the name of the sub fragment
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="extractCodeFrag">
		<xsl:param name="fragName"/>
		<xsl:param name="codeFrag"/>
		<xsl:value-of select="substring-before(substring-after($codeFrag,concat('// Begin ',$fragName)),concat('// End ',$fragName))"/>
	</xsl:template>
	
	<!-- 
	***************************************************************************************************************************************************************************
	Fills in a Property
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="writeProperty">
		<xsl:param name="codeFrag"/>
		<xsl:param name="propType"/>
			<Property>
				<xsl:attribute name="Name">
					<xsl:value-of select="$propType"/>
				</xsl:attribute>
				<xsl:variable name="propVal">
					<xsl:call-template name="extractCodeFrag">
						<xsl:with-param name="fragName" select="$propType"/>
						<xsl:with-param name="codeFrag" select="$codeFrag"/>
					</xsl:call-template>
				</xsl:variable>
				<Attributes>
					<xsl:call-template name="writeAttribute">
						<xsl:with-param name="attribLines">
							<xsl:value-of select="$propVal"/>
						</xsl:with-param>
						<xsl:with-param name="propType">
							<xsl:value-of select="$propType"/>
						</xsl:with-param>
					</xsl:call-template>
				</Attributes>
			</Property>
	</xsl:template>
	
	<!-- 
	***************************************************************************************************************************************************************************
	Fills in the TemplateDefintion Properties
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="writeProperties">
		<xsl:param name="codeFrag"/>
		<Properties>
			<xsl:call-template name="writeProperty">
				<xsl:with-param name="codeFrag" select="$codeFrag"/>
				<xsl:with-param name="propType" select="'PropertyValues'"/>
			</xsl:call-template>
			<xsl:call-template name="writeProperty">
				<xsl:with-param name="codeFrag" select="$codeFrag"/>
				<xsl:with-param name="propType" select="'Imports'"/>
			</xsl:call-template>
			<xsl:call-template name="writeProperty">
				<xsl:with-param name="codeFrag" select="$codeFrag"/>
				<xsl:with-param name="propType" select="'Declaration'"/>
			</xsl:call-template>
		</Properties>
	</xsl:template>
	<!-- 
	***************************************************************************************************************************************************************************
	Fills in the GeneratedVars
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="writeGeneratedVarsWithCodeFrag">
		<xsl:param name="codeFrag"/>
		<xsl:call-template name="extractCodeFrag">
			<xsl:with-param name="fragName" select="'GeneratedVars'"/>
			<xsl:with-param name="codeFrag" select="$codeFrag"/>
		</xsl:call-template>
	</xsl:template>
	
	<xsl:template match="bpel:container" mode="GeneratedVars">
		<xsl:variable name="BusObjType">
			<xsl:call-template name="getBusObjType">
				<xsl:with-param name="msgType" select="@messageType"/>
			</xsl:call-template>
		</xsl:variable>
		BusObj <xsl:value-of select="@name"/>BusObj = new BusObj("<xsl:value-of select="$BusObjType"/>");
	</xsl:template>
	
	<xsl:template name="writeGeneratedVars">
		<xsl:apply-templates select="bpel:process/bpel:containers/bpel:container" mode="GeneratedVars"/>
	</xsl:template>

	<!-- 
	***************************************************************************************************************************************************************************
	Gets the type of the BusObj
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="getBusObjType">
		<xsl:param name="msgType"/>
		<xsl:variable name="msgTypeNoNSPrefix" select="substring-after($msgType,':')"/>
		<xsl:variable name="msgTypeNoMsgSuffix" select="substring-before($msgTypeNoNSPrefix,'Msg')"/>
		<xsl:choose>
			<xsl:when test="boolean(normalize-space($msgTypeNoMsgSuffix))">		
				<xsl:value-of select="$msgTypeNoMsgSuffix"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$msgTypeNoNSPrefix"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<!-- 
	***************************************************************************************************************************************************************************
	Fills in the BusObjRefs
	***************************************************************************************************************************************************************************
-->
	<xsl:template match="bpel:container" mode="BusObjRefs">
		<xsl:param name="scenarios"/>
		<xsl:variable name="containerName" select="@name"/>
		<BusObjRef>
			<xsl:attribute name="Name"><xsl:value-of select="$containerName"/></xsl:attribute>
			<xsl:attribute name="BusObjType">
				<xsl:call-template name="getBusObjType">
					<xsl:with-param name="msgType" select="@messageType"/>
				</xsl:call-template>
			</xsl:attribute>
			<xsl:attribute name="IsRequired">Yes</xsl:attribute>
			<xsl:attribute name="IsConfigured">No</xsl:attribute>
			<ConnectorName/>
			<Verbs>
				<!-- creates verbs based on pick -->
				<xsl:for-each select="//bpel:pick/bpel:onMessage[@container=$containerName]">
					<Verb>
						<xsl:attribute name="Name"><xsl:value-of select="@operation"/></xsl:attribute>
						<xsl:attribute name="IsSubscribeEvent">Yes</xsl:attribute>
						<ScenarioName>
							<xsl:variable name="sourceLinkName" select="bpel:sequence/bpel:empty/bpel:source/@linkName"/>
							<xsl:value-of select="/bpel:process/bpel:flow/bpel:flow[.//bpel:target/@linkName=$sourceLinkName]/@name"/>
						</ScenarioName>
					</Verb>
				</xsl:for-each>
				<!-- creates verbs based on retreive -->
				<!--
				<xsl:for-each select="//bpel:receive[(@container=$containerName)and(@createInstance='yes')]">
					<Verb>
						<xsl:attribute name="Name"><xsl:value-of select="@operation"/></xsl:attribute>
						<xsl:attribute name="IsSubscribeEvent">Yes</xsl:attribute>
						<ScenarioName>
							<xsl:variable name="desc" select="concat('partner=',@partner,' portType=',@portType,' operation=',@operation)"/>
							<xsl:value-of select="xalan:nodeset($scenarios)//Scenario[.//Desc=$desc]/@Name"/>
						</ScenarioName>
					</Verb>
				</xsl:for-each>
				-->
			</Verbs>
		</BusObjRef>	
	</xsl:template>
	
	<!-- 
	***************************************************************************************************************************************************************************
	Writes BusObjRefs based on Containers
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="writeBusObjRefs">
		<xsl:param name="scenarios"/>
		<xsl:choose>
			<xsl:when test="$CWBPEL">
				<xsl:apply-templates select="/bpel:process/bpel:containers/bpel:container" mode="BusObjRefs">
					<xsl:with-param name="scenarios" select="$scenarios"/>
				</xsl:apply-templates>
			</xsl:when>
			<xsl:otherwise>
				<!-- find all ports in the scenario -->
				<xsl:variable name="allServiceNodes" select="xalan:nodeset($scenarios)//ServiceNode"/>
				<xsl:variable name="allContainers" select="/bpel:process/bpel:containers"/>
				<xsl:for-each select="$allServiceNodes">
					<xsl:variable name="busObjName" select="substring-before(.//RegBusObjName,'BusObj')"/>
					<xsl:variable name="busObjType"> 
						<xsl:call-template name="getBusObjType">
							<xsl:with-param name="msgType" select="$allContainers/bpel:container[@name=$busObjName]/@messageType"/>
						</xsl:call-template>
					</xsl:variable>
					<BusObjRef>
						<xsl:attribute name="Name"><xsl:value-of select=".//RegBusObjRefName"/></xsl:attribute>
						<xsl:attribute name="BusObjType"><xsl:value-of select="$busObjType"/></xsl:attribute>
						<xsl:attribute name="IsRequired">Yes</xsl:attribute>
						<xsl:attribute name="IsConfigured">No</xsl:attribute>
						<ConnectorName/>
						<Verbs>
							<!-- creates verbs based on pick -->
							<!--
							<xsl:for-each select="//bpel:receive[(@container=$containerName)and(@createInstance='yes')]">
								<Verb>
									<xsl:attribute name="Name"><xsl:value-of select="@operation"/></xsl:attribute>
									<xsl:attribute name="IsSubscribeEvent">Yes</xsl:attribute>
									<ScenarioName>
										<xsl:variable name="desc" select="concat('partner=',@partner,' portType=',@portType,' operation=',@operation)"/>
										<xsl:value-of select="xalan:nodeset($scenarios)//Scenario[.//Desc=$desc]/@Name"/>
									</ScenarioName>
								</Verb>
							</xsl:for-each>
							-->
						</Verbs>
					</BusObjRef>					
				</xsl:for-each>
				<xsl:variable name="allReceive" select="//bpel:receive"/>
				<!-- Create From ports -->
				<xsl:for-each select="$allContainers/bpel:container">
					<xsl:variable name="containerName" select="@name"/>
					<xsl:variable name="busObjType"> 
						<xsl:call-template name="getBusObjType">
							<xsl:with-param name="msgType" select="@messageType"/>
						</xsl:call-template>
					</xsl:variable>
					<xsl:for-each select="$allReceive[(@container=$containerName)and(@createInstance='yes')]">
						<BusObjRef>
							<xsl:attribute name="Name"><xsl:value-of select="concat('From','-',$containerName,'-Port')"/></xsl:attribute>
							<xsl:attribute name="BusObjType"><xsl:value-of select="$busObjType"/></xsl:attribute>
							<xsl:attribute name="IsRequired">Yes</xsl:attribute>
							<xsl:attribute name="IsConfigured">No</xsl:attribute>
							<ConnectorName/>
							<Verbs>
								<Verb>
									<xsl:attribute name="Name"><xsl:value-of select="@operation"/></xsl:attribute>
									<xsl:attribute name="IsSubscribeEvent">Yes</xsl:attribute>
									<ScenarioName>
										<xsl:variable name="desc" select="concat('partner=',@partner,' portType=',@portType,' operation=',@operation)"/>
										<xsl:value-of select="xalan:nodeset($scenarios)//Scenario[.//Desc=$desc]/@Name"/>
									</ScenarioName>
								</Verb>
							</Verbs>
						</BusObjRef>					
					</xsl:for-each>
				</xsl:for-each>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!-- 
	***************************************************************************************************************************************************************************
	Navigate a Collab Scenarios
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="navigateScenarios">
		<xsl:choose>
			<xsl:when test="$CWBPEL">
				<xsl:apply-templates select="bpel:process/bpel:flow/bpel:flow" mode="Scenario"/>
			</xsl:when>
			<xsl:otherwise>
				<!-- Assume the BPEL always uses a structured activity i.e. sequence or flow -->
				<xsl:apply-templates select="bpel:process/bpel:sequence" mode="Scenario"/>
				<xsl:apply-templates select="bpel:process/bpel:flow" mode="Scenario"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!-- 
	***************************************************************************************************************************************************************************
	Fills in the Collab Scenarios
	***************************************************************************************************************************************************************************
-->	
	<xsl:template name="writeScenario">
		<xsl:param name="type"/>
		<xsl:param name="currentPath"/>
		<Scenario>
			<xsl:attribute name="Name"><xsl:value-of select="$currentPath"/></xsl:attribute>
			<xsl:attribute name="Version"><xsl:value-of select="$Version"/></xsl:attribute>
			<xsl:attribute name="DiagramRef"><xsl:value-of select="$currentPath"/></xsl:attribute>
			<xsl:attribute name="DiagramRefGUI">GUI__<xsl:value-of select="$currentPath"/></xsl:attribute>
			<xsl:call-template name="writeHeader"/>
			<GUIDefinition>
				<Desc/> 
				<GUIUserSection1/> 
				<GUIUserSection2/> 
				<GUIVariables/> 
				</GUIDefinition>			
			<GUIDiagrams/>
			<Desc/>
			<ModelScenario>
				<xsl:call-template name="writeDiagram">
					<xsl:with-param name="name" select="$currentPath"/>
					<xsl:with-param name="type" select="$type"/>
					<xsl:with-param name="currentPath" select="$currentPath"/>
				</xsl:call-template>
				<!-- Continue to process all subdiagram and iteration diagram -->
				<!-- need to call templates in sequence -->
				<xsl:for-each select="child::*">
					<xsl:variable name="nodeType" select="name(.)"/>
					<xsl:if test="$nodeType='sequence' or $nodeType='flow' or $nodeType='while'">
						<xsl:apply-templates select="." mode="Diagram">
							<xsl:with-param name="currentPath" select="$currentPath"/>
						</xsl:apply-templates>
					</xsl:if>
				</xsl:for-each>
			</ModelScenario>
		</Scenario>
	</xsl:template>
	<!-- 
	***************************************************************************************************************************************************************************
	Fills in the Collab Scenarios for 'flow'
	***************************************************************************************************************************************************************************
-->
	<xsl:template match="bpel:flow" mode="Scenario">
		<xsl:variable name="currentPath">
			<xsl:call-template name="getUniqueName">
				<xsl:with-param name="name" select="@name"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:call-template name="writeScenario">
			<xsl:with-param name="type" select="'flow'"/>
			<xsl:with-param name="currentPath" select="$currentPath"/>
		</xsl:call-template>
	</xsl:template>
	<!-- 
	***************************************************************************************************************************************************************************
	Fills in the Collab Scenarios for 'sequence'
	***************************************************************************************************************************************************************************
-->
	<xsl:template match="bpel:sequence" mode="Scenario">
		<xsl:variable name="currentPath">
			<xsl:call-template name="getUniqueName">
				<xsl:with-param name="name" select="@name"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:call-template name="writeScenario">
			<xsl:with-param name="type" select="'sequence'"/>
			<xsl:with-param name="currentPath" select="$currentPath"/>
		</xsl:call-template>
	</xsl:template>

	<!-- 
	***************************************************************************************************************************************************************************
	Utility template - Get unique id
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="getUniqueID">
		<xsl:param name="name"/>
		<xsl:variable name="id">
			<xsl:variable name="_id" select="java:getID($uniqueID)"/>
			<!-- need to truncate the id to 8 digits.  This is a work around to the problem of the
				 CW ProcessDesigner where it cannot show the name/label of the SubDiagram node if the
				 UniqID is too long
			 -->
			<xsl:variable name="idLength" select="string-length($_id)"/>
			<xsl:variable name="truncatedID" select="substring($_id,number($idLength)-7)"/>
			<!-- there could be a chance that the returned id is e.g. 100000000.  After it is truncated
				the id becomes 00000000.  The chance of this happens is pretty rare though.  However, just in case,
				the following 'when' code is to protect this problem
			-->
			<xsl:choose>
				<xsl:when test="substring($truncatedID,1,1)='0'">
					<xsl:value-of select="concat('1',substring($truncatedID,2))"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="$truncatedID"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:value-of select="concat($name,'_',$id)"/>
	</xsl:template>

	<!-- 
	***************************************************************************************************************************************************************************
	Utility template - Get unique name
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="getUniqueName">
		<xsl:param name="name"/>
		<xsl:choose>
			<xsl:when test="boolean(normalize-space($name))">
				<xsl:value-of select="$name"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:variable name="uniqID">
					<xsl:call-template name="getUniqueID"/>
				</xsl:variable>
				<xsl:value-of select="concat(name(.),substring-after($uniqID,'_'))"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- 
	***************************************************************************************************************************************************************************
	Fills in the Collab Scenarios SubDiagrams
	***************************************************************************************************************************************************************************
-->
	<xsl:template match="bpel:flow" mode="Diagram">
		<xsl:param name="currentPath"/>
		<xsl:choose>
			<!-- if 'flow' has other activities other than 'while' then treat it as a sub-diagram.  Otherwise
				if it has only 'while' then treat it as an iteration-diagram
			-->
			<xsl:when test="boolean(bpel:assign) or boolean(bpel:empty) or boolean(bpel:flow) or boolean(bpel:throw) or boolean(bpel:sequence) or boolean(bpel:invoke) or boolean(bpel:receive) or boolean(bpel:reply) or boolean(bpel:switch)">
				<!-- obtain the name of the subDiagram by retrieving a list of previously stored names when
					processing the structured activity
				 -->
				<xsl:variable name="newPath">
						<xsl:variable name="subDiagRefs" select="java:get($simpleStrVar,$currentPath)"/>
						<xsl:variable name="subDiagRef" select="substring-before($subDiagRefs,';')"/>
						<xsl:variable name="setSubDiagRef" select="java:set($simpleStrVar,$currentPath,substring-after($subDiagRefs,';'))"/>
						<xsl:choose>
							<xsl:when test="boolean(normalize-space($subDiagRef))">
								<xsl:value-of select="$subDiagRef"/>
							</xsl:when>
							<xsl:otherwise>
								<xsl:if test="$CWBPEL">
									<xsl:call-template name="getUniqueName">
										<xsl:with-param name="name" select="@name"/>
									</xsl:call-template>
								</xsl:if>
							</xsl:otherwise>
						</xsl:choose>	
				</xsl:variable>
				<xsl:call-template name="writeDiagram">
					<xsl:with-param name="name" select="$newPath"/>
					<xsl:with-param name="type" select="'flow'"/>
					<xsl:with-param name="currentPath" select="$newPath"/>
				</xsl:call-template>
				<!-- need to call templates in sequence -->
				<xsl:for-each select="child::*">
					<xsl:variable name="nodeType" select="name(.)"/>
					<xsl:if test="$nodeType='sequence' or $nodeType='flow' or $nodeType='while'">
						<xsl:apply-templates select="." mode="Diagram">
							<xsl:with-param name="currentPath" select="$newPath"/>
						</xsl:apply-templates>
					</xsl:if>
				</xsl:for-each>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="bpel:while" mode="Diagram">
						<xsl:with-param name="currentPath" select="$currentPath"/>
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!-- 
	***************************************************************************************************************************************************************************
	Fills in the Collab Scenarios Iterator Nodes
	***************************************************************************************************************************************************************************
-->
	<xsl:template match="bpel:while" mode="Diagram">
		<xsl:param name="currentPath"/>
		<!-- only support a structured activity including flow or sequence within a while
		-->
		<xsl:apply-templates select="bpel:flow" mode="Diagram">
			<xsl:with-param name="currentPath" select="$currentPath"/>
		</xsl:apply-templates>
		<xsl:apply-templates select="bpel:sequence" mode="Diagram">
			<xsl:with-param name="currentPath" select="$currentPath"/>
		</xsl:apply-templates>
	</xsl:template>
	<!-- 
	***************************************************************************************************************************************************************************
	Fills in a Diagram within a Scenario
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="writeDiagram">
		<xsl:param name="name"/>
		<xsl:param name="type"/>
		<xsl:param name="currentPath"/>
		<Diagram>
			<xsl:attribute name="Name"><xsl:value-of select="$name"/></xsl:attribute>
			<xsl:attribute name="Version"><xsl:value-of select="$Version"/></xsl:attribute>
			<xsl:call-template name="writeModelStructureElement">
				<xsl:with-param name="name" select="$name"/>
			</xsl:call-template>
			<xsl:call-template name="writeDeclaration"/>
			<xsl:call-template name="writeDiagramBody">
				<xsl:with-param name="name" select="$name"/>
				<xsl:with-param name="type" select="$type"/>
				<xsl:with-param name="currentPath" select="$currentPath"/>
			</xsl:call-template>
		</Diagram>
	</xsl:template>
	<!-- 
	***************************************************************************************************************************************************************************
	Fills in a Diagram ModelStructureElement
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="writeModelStructureElement">
		<xsl:param name="name"/>
		<xsl:variable name="ModelName">
			<xsl:call-template name="findLastElement">
				<xsl:with-param name="name">
					<xsl:value-of select="$name"/>
				</xsl:with-param>
				<xsl:with-param name="delimiter">.</xsl:with-param>
			</xsl:call-template>
		</xsl:variable>
		<ModelStructureElement>
			<xsl:attribute name="Name"><xsl:value-of select="$ModelName"/></xsl:attribute>
			<ModelElement>
				<xsl:attribute name="Version"><xsl:value-of select="$ModelElementVersion"></xsl:value-of></xsl:attribute>
				<Desc/>
			</ModelElement>
		</ModelStructureElement>
	</xsl:template>
	<!-- 
	***************************************************************************************************************************************************************************
	Returns the last Element of a String delimited with $delimiter
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="findLastElement">
		<xsl:param name="name"/>
		<xsl:param name="delimiter"/>
		<xsl:variable name="SecondElement" select="substring-after($name,$delimiter)"/>
		<xsl:choose>
			<xsl:when test="boolean(normalize-space($SecondElement))">
				<xsl:call-template name="findLastElement">
					<xsl:with-param name="name">
						<xsl:value-of select="$SecondElement"/>
					</xsl:with-param>
					<xsl:with-param name="delimiter">
						<xsl:value-of select="$delimiter"/>
					</xsl:with-param>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$name"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!-- 
	***************************************************************************************************************************************************************************
	Returns all but last Element of a String delimited with $delimiter
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="findAllButLastElement">
		<xsl:param name="name"/>
		<xsl:param name="delimiter"/>
		<xsl:param name="returnData"/>
		<xsl:variable name="FirstElement" select="substring-before($name,$delimiter)"/>
		<xsl:variable name="SecondElement" select="substring-after($name,$delimiter)"/>
		<xsl:variable name="retData">
			<xsl:choose>
				<xsl:when test="boolean(normalize-space($returnData))">
					<xsl:value-of select="concat($returnData,$delimiter,$FirstElement)"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="concat($returnData,$FirstElement)"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:choose>
			<xsl:when test="contains($SecondElement,$delimiter)">
				<xsl:call-template name="findAllButLastElement">
					<xsl:with-param name="name">
						<xsl:value-of select="$SecondElement"/>
					</xsl:with-param>
					<xsl:with-param name="delimiter">
						<xsl:value-of select="$delimiter"/>
					</xsl:with-param>
					<xsl:with-param name="returnData">
						<xsl:value-of select="$retData"/>
					</xsl:with-param>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$retData"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!-- 
	***************************************************************************************************************************************************************************
	Fills in a Diagram Declaration
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="writeDeclaration">
		<Declaration>
			<Section1>
			<xsl:value-of select="wswf:script[not(boolean(normalize-space(@name)))]"/>
			</Section1>
			<Section2/>
		</Declaration>
	</xsl:template>

	<!-- 
	***************************************************************************************************************************************************************************
	Detects initial link - for CW BPEL only.
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="detectInitialLink">
		<!-- find if there is a node where its target link is not found in the links.  That means it must
			be linked with one of the pick message -->
		<xsl:for-each select="child::*">
			<xsl:if test="boolean(./bpel:target)">
				<xsl:if test="not(./bpel:target/@linkName=../bpel:links/bpel:link/@name)">
					<xsl:call-template name="link">
						<xsl:with-param name="linkParam" select="./bpel:target/@linkName"/>
					</xsl:call-template>
				</xsl:if>
			</xsl:if>
		</xsl:for-each>
	</xsl:template>
	
	<!-- 
	***************************************************************************************************************************************************************************
	Fills in a Diagram Body
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="writeDiagramBody">
		<xsl:param name="name"/>
		<xsl:param name="type"/>
		<xsl:param name="currentPath"/>
		<xsl:variable name="id">
			<xsl:call-template name="findLastElement">
				<xsl:with-param name="name">
					<xsl:value-of select="$name"/>
				</xsl:with-param>
				<xsl:with-param name="delimiter">.</xsl:with-param>
			</xsl:call-template>
		</xsl:variable>
		<DiagramBody>
			<xsl:call-template name="writeDiagramModelActivityElement">
				<xsl:with-param name="name" select="$name"/>
			</xsl:call-template>
			<xsl:choose>	
				<xsl:when test="$CWBPEL">
					<xsl:call-template name="writeNodes"/>
					<Links>
						<xsl:apply-templates select="bpel:links/bpel:link"/>
						<xsl:call-template name="detectInitialLink"/>
					</Links>
				</xsl:when>
				<xsl:otherwise>
					<!-- if the current processing node is a 'flow' then call processFlow template
						Otherwise call processSequence.
					-->
					<xsl:choose>
						<xsl:when test="$type='flow'">
							<xsl:call-template name="processFlow">
								<xsl:with-param name="currentPath" select="$currentPath"/>
							</xsl:call-template>
						</xsl:when>
						<xsl:otherwise>
							<xsl:call-template name="processSequence">
								<xsl:with-param name="currentPath" select="$currentPath"/>
							</xsl:call-template>
						</xsl:otherwise>						
					</xsl:choose>
				</xsl:otherwise>
			</xsl:choose>
		</DiagramBody>
	</xsl:template>
	<!-- 
	***************************************************************************************************************************************************************************
	Fills in a Nodes ModelActivityElement
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="writeDiagramModelActivityElement">
		<xsl:param name="name"/>
		<xsl:variable name="delimiter" select="'.'"/>
		<xsl:variable name="requiresUniqueID" select="contains($name,$delimiter)"/>
		<xsl:variable name="id">
			<xsl:choose>
				<xsl:when test="boolean($requiresUniqueID)">
					<xsl:call-template name="findLastElement">
						<xsl:with-param name="name">
							<xsl:value-of select="$name"/>
						</xsl:with-param>
						<xsl:with-param name="delimiter">.</xsl:with-param>
					</xsl:call-template>
				</xsl:when>
				<xsl:otherwise>
					<xsl:call-template name="getUniqueID">
						<xsl:with-param name="name" select="$name"/>			
					</xsl:call-template>
				</xsl:otherwise>			
			</xsl:choose>
		</xsl:variable>
		<ModelActivityElement>
			<xsl:choose>
				<xsl:when test="contains($id,'_')">
					<xsl:attribute name="UniqID"><xsl:value-of select="$id"/></xsl:attribute>
				</xsl:when>
				<xsl:otherwise>
					<xsl:attribute name="UniqID"><xsl:value-of select="$id"/>_-1</xsl:attribute>
				</xsl:otherwise>
			</xsl:choose>
			<ModelElement>
				<xsl:attribute name="Version"><xsl:value-of select="$ModelElementVersion"></xsl:value-of></xsl:attribute>
				<Desc/>
			</ModelElement>
			<Label/>
		</ModelActivityElement>
	</xsl:template>
	<!-- 
	***************************************************************************************************************************************************************************
	Fills in a Nodes ModelActivityElement
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="writeNodeModelActivityElement">
		<xsl:param name="name"/>
		<xsl:param name="desc"/>
		<xsl:variable name="delimiter" select="'_'"/>
		<xsl:variable name="requiresUniqueID" select="contains($name,$delimiter)"/>
		<xsl:variable name="id">
			<xsl:choose>
				<xsl:when test="boolean($requiresUniqueID)">
					<xsl:call-template name="findLastElement">
						<xsl:with-param name="name">
							<xsl:value-of select="$name"/>
						</xsl:with-param>
						<xsl:with-param name="delimiter">_</xsl:with-param>
					</xsl:call-template>
				</xsl:when>
				<xsl:otherwise>
					<xsl:call-template name="getUniqueID">
						<xsl:with-param name="name" select="''"/>			
					</xsl:call-template>
				</xsl:otherwise>		
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="label">
			<xsl:variable name="tempLabel">
				<xsl:call-template name="findAllButLastElement">
					<xsl:with-param name="name">
						<xsl:value-of select="$name"/>
					</xsl:with-param>
					<xsl:with-param name="delimiter">_</xsl:with-param>
				</xsl:call-template>
			</xsl:variable>
			<xsl:choose>
				<xsl:when test="boolean(normalize-space($tempLabel))">
					<xsl:value-of select="$tempLabel"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="$name"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<ModelActivityElement>
			<xsl:choose>
				<xsl:when test="boolean($requiresUniqueID)">
					<xsl:attribute name="UniqID">_<xsl:value-of select="$id"/></xsl:attribute>
				</xsl:when>
				<xsl:otherwise>
					<xsl:attribute name="UniqID"><xsl:value-of select="$id"/></xsl:attribute>
				</xsl:otherwise>		
			</xsl:choose>
			<ModelElement>
				<xsl:attribute name="Version"><xsl:value-of select="$ModelElementVersion"></xsl:value-of></xsl:attribute>
				<Desc>
					<xsl:value-of select="$desc"/>
				</Desc>
			</ModelElement>
			<Label>
				<xsl:value-of select="$label"/>
			</Label>
		</ModelActivityElement>
	</xsl:template>

	<!-- 
	***************************************************************************************************************************************************************************
	Fills in an Exception link
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="writeExceptionLink">
		<xsl:param name="source"/>
		<xsl:param name="target"/>
		<xsl:param name="id"/>
		<ExceptionLink>
			<xsl:call-template name="writeLink">
				<xsl:with-param name="source" select="$source"/>
				<xsl:with-param name="target" select="$target"/>
				<xsl:with-param name="id" select="$id"/>
			</xsl:call-template>
			<ExceptionName>AnyException</ExceptionName>
		</ExceptionLink>
	</xsl:template>	

	<!-- 
	***************************************************************************************************************************************************************************
	Fills in a Message link
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="writeMessageLink">
		<xsl:param name="source"/>
		<xsl:param name="target"/>
		<xsl:param name="id"/>
		<MessageLink>
			<xsl:call-template name="writeLink">
				<xsl:with-param name="source" select="$source"/>
				<xsl:with-param name="target" select="$target"/>
				<xsl:with-param name="id" select="$id"/>
			</xsl:call-template>
		</MessageLink>
	</xsl:template>	

	<!-- 
	***************************************************************************************************************************************************************************
	Fills in a Transition link
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="writeTransitionLink">
		<xsl:param name="source"/>
		<xsl:param name="target"/>
		<xsl:param name="id"/>
		<TransitionLink>
			<xsl:call-template name="writeLink">
				<xsl:with-param name="source" select="$source"/>
				<xsl:with-param name="target" select="$target"/>
				<xsl:with-param name="id" select="$id"/>
			</xsl:call-template>
			<GuardCondition/>
		</TransitionLink>
	</xsl:template>	
	<!-- 
	***************************************************************************************************************************************************************************
	Fills in a Diagram link
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="link" match="bpel:link">
		<xsl:param name="linkParam"/>
		<xsl:variable name="linkName">		
			<xsl:choose>
				<xsl:when test="boolean(normalize-space($linkParam))">
					<xsl:value-of select="$linkParam"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="@name"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="id" select="substring-before($linkName,':')"/>
		<xsl:variable name="linkStr" select="substring-after($linkName,':')"/>
		<xsl:variable name="source" select="substring-before($linkStr,'-to-')"/>
		<xsl:variable name="sourcetemp">
			<xsl:call-template name="findLastElement">
				<xsl:with-param name="name">
					<xsl:value-of select="$source"/>
				</xsl:with-param>
				<xsl:with-param name="delimiter">_</xsl:with-param>
			</xsl:call-template>				
		</xsl:variable>
		<xsl:variable name="target" select="substring-after($linkStr,'-to-')"/>
		<xsl:variable name="targettemp">
			<xsl:call-template name="findLastElement">
				<xsl:with-param name="name">
					<xsl:value-of select="$target"/>
				</xsl:with-param>
				<xsl:with-param name="delimiter">_</xsl:with-param>
			</xsl:call-template>				
		</xsl:variable>
		<!-- check if the link is an exception, message, or transition link -->
		<xsl:choose>
			<xsl:when test="../../bpel:throw/bpel:target[@linkName=$linkName]">
				<xsl:call-template name="writeExceptionLink">
					<xsl:with-param name="source" select="$source"/>
					<xsl:with-param name="target" select="$target"/>
					<xsl:with-param name="id" select="$id"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="../../bpel:invoke/bpel:target[@linkName=$linkName]">
				<xsl:call-template name="writeMessageLink">
					<xsl:with-param name="source" select="$source"/>
					<xsl:with-param name="target" select="$target"/>
					<xsl:with-param name="id" select="$id"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name="writeTransitionLink">
					<xsl:with-param name="source" select="$source"/>
					<xsl:with-param name="target" select="$target"/>
					<xsl:with-param name="id" select="$id"/>
				</xsl:call-template>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!-- 
	***************************************************************************************************************************************************************************
	Fills in a link content
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="writeLink">
		<xsl:param name="source"/>
		<xsl:param name="target"/>
		<xsl:param name="id"/>
		<xsl:variable name="sourceid">
			<xsl:call-template name="findLastElement">
				<xsl:with-param name="name">
					<xsl:value-of select="$source"/>
				</xsl:with-param>
				<xsl:with-param name="delimiter">_</xsl:with-param>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="targetid">
			<xsl:call-template name="findLastElement">
				<xsl:with-param name="name">
					<xsl:value-of select="$target"/>
				</xsl:with-param>
				<xsl:with-param name="delimiter">_</xsl:with-param>
			</xsl:call-template>
		</xsl:variable>
		<Link>
			<xsl:call-template name="writeNodeModelActivityElement">
				<xsl:with-param name="name">
					<xsl:value-of select="$id"/>
				</xsl:with-param>
			</xsl:call-template>
			<Connect>
				<xsl:attribute name="To">_<xsl:value-of select="$targetid"/></xsl:attribute>
				<xsl:attribute name="From">_<xsl:value-of select="$sourceid"/></xsl:attribute>
			</Connect>
		</Link>
	</xsl:template>

	<!-- 
	***************************************************************************************************************************************************************************
	Add a dummy action node if the currently processing node is a service node
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="fixActionNode">
		<xsl:param name="inputNode"/>
		<!-- If the input node is a service node, then we have to create a dummy action node that
			 acts as an intermediate node that links the previous node and the service node
			 because CrossWorlds only allows an action node to link to a service node.
		-->
	    <xsl:if test="contains(name(xalan:nodeset($inputNode)/*[position()=1]),'ServiceNode')">
			<!-- Creates dummy action node -->
			<xsl:variable name="actionNode">
				<xsl:call-template name="writeActionNode"/>
			</xsl:variable>
			<!-- Set next element as the dummy action node -->
			<xsl:copy-of select="$actionNode"/>
			<!-- Create Link between dummy action and service node -->
			<xsl:variable name="source" select="xalan:nodeset($actionNode)[position()=1]//ModelActivityElement/@UniqID"/>
			<xsl:variable name="target" select="xalan:nodeset($inputNode)[position()=1]//ModelActivityElement/@UniqID"/>
			<xsl:variable name="Link">
				<xsl:call-template name="createLink">
					<xsl:with-param name="source" select="$source"/>
					<xsl:with-param name="target" select="$target"/>
					<xsl:with-param name="targetType" select="name(xalan:nodeset($inputNode)/*[position()=1])"/>
				</xsl:call-template>
			</xsl:variable>
			<xsl:variable name="newLinks" select="java:appendNodeAsString($nodeSetVar,$linksKey,$Link)"/>		    			
		</xsl:if>
		<xsl:copy-of select="$inputNode"/>
	</xsl:template>
	
	<!-- 
	***************************************************************************************************************************************************************************
	Process Flow activity
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="processFlow">
    	<xsl:param name="currentPath"/>
   		<xsl:variable name="resetLinks" select="java:reset($nodeSetVar,$linksKey)"/>		    			
		<xsl:variable name="allLinks" select="bpel:links"/>
    	<Nodes>    	
	    	<!-- Create initial node -->
	    	<xsl:variable name="initialNode">
	   			<xsl:call-template name="writeInitialNode"/>
			</xsl:variable>
			<xsl:copy-of select="$initialNode"/>
	    	<!-- Create Success End Node -->
	    	<xsl:variable name="successEndNode"> 
				<xsl:call-template name="writeSuccessEndNode"/>
			</xsl:variable>
			<xsl:copy-of select="$successEndNode"/>
			<!-- Create nodes for activities found within the flow -->
	    	<xsl:for-each select="child::*">
	    		<xsl:variable name="currentNode">
	    			<xsl:call-template name="createElement">
	    				<xsl:with-param name="elem" select="."/>
	    				<xsl:with-param name="currentPath" select="$currentPath"/>
	    			</xsl:call-template>
	    		</xsl:variable>
	    		<xsl:variable name="fixedNode">
	   				<xsl:call-template name="fixActionNode">
	   					<xsl:with-param name="inputNode" select="$currentNode"/>
	   				</xsl:call-template>			    
				</xsl:variable>
				<xsl:copy-of select="$fixedNode"/>
				<xsl:variable name="workingNode" select="xalan:nodeset($fixedNode)[position()=1]"/>
				<xsl:variable name="workingNodeName" select="name(xalan:nodeset($workingNode)/*[position()=1])"/>
	    		<!-- If the node is a SubDiagram or Iterator node, then we need to keep track of the 
	    			 name of the subdiagram reference so that when later on the diagram node is being created 
	    			 for the subdiagram, we have to make sure the same name is used in the Diagram node and matches the 
	    			 DiagramRef in the SubDiagram node.  This is useful if the name is being generated
	    			 in the case that there is no name specified in 'sequence' or 'flow' tab which represents
	    			 a subdiagram.
	    		-->
	    		<xsl:if test="$workingNodeName='SubDiagramNode' or $workingNodeName='IteratorNode'">
	    			<xsl:variable name="subDiagRef" select="xalan:nodeset($workingNode)/*[position()=1]/CompositeNode/@DiagramRef"/>
	    			<xsl:variable name="subDiagRefListVar" select="java:get($simpleStrVar,$currentPath)"/>
	    			<xsl:variable name="setSubDiagRefList" select="java:set($simpleStrVar,$currentPath,concat($subDiagRefListVar,$subDiagRef,';'))"/>
	    		</xsl:if>
	    		<!-- Find the names of the links that this node represent within the flow -->
				<xsl:variable name="sourceLinkNames" select=".//bpel:source[@linkName=$allLinks/bpel:link/@name]"/>
				<xsl:variable name="targetLinkNames" select=".//bpel:target[@linkName=$allLinks/bpel:link/@name]"/>
				<xsl:variable name="allLinkNames" select="$sourceLinkNames|$targetLinkNames"/>
				<xsl:if test="name(.)!='links' and name(.)!='source' and name(.)!='target'">
					<!-- store the node using linkNames as key to create links later on -->
					<xsl:for-each select="$allLinkNames">
						<xsl:variable name="keyName" select="concat(name(.),./@linkName)"/>
						<xsl:variable name="setNodeForLink" select="java:set($nodeSetVar,$keyName,$workingNode)"/>
					</xsl:for-each>
					<xsl:if test="not(boolean($targetLinkNames))">
						<!-- Create link from initial node to this node -->
						<xsl:variable name="initialLink">
			   				<xsl:call-template name="createLink">
		   						<xsl:with-param name="source" select="xalan:nodeset($initialNode)[position()=1]//ModelActivityElement/@UniqID"/>
		   						<xsl:with-param name="target" select="xalan:nodeset($workingNode)[position()=1]//ModelActivityElement/@UniqID"/>
								<xsl:with-param name="targetType" select="$workingNodeName"/>   					
		   					</xsl:call-template>
	   					</xsl:variable>
						<xsl:variable name="appendLinks" select="java:appendNodeAsString($nodeSetVar,$linksKey,$initialLink)"/>		    									
					</xsl:if>
					<xsl:variable name="nodeName" select="name(xalan:nodeset($workingNode)[position()=1])"/>
					<xsl:if test="not(boolean($sourceLinkNames)) and $nodeName != 'SuccessEndNode' and $nodeName != 'FailureEndNode'">
						<!-- Create link from this node to end node -->
						<xsl:variable name="successEndLink">
			   				<xsl:call-template name="createLink">
		   						<xsl:with-param name="source" select="xalan:nodeset($workingNode)[position()=1]//ModelActivityElement/@UniqID"/>
		   						<xsl:with-param name="target" select="xalan:nodeset($successEndNode)[position()=1]//ModelActivityElement/@UniqID"/>
								<xsl:with-param name="targetType" select="name(xalan:nodeset($successEndNode)/*[position()=1])"/>   					
		   					</xsl:call-template>					
	   					</xsl:variable>
						<xsl:variable name="appendLinks" select="java:appendNodeAsString($nodeSetVar,$linksKey,$successEndLink)"/>		    			
					</xsl:if>
				</xsl:if>
			</xsl:for-each>						
	    </Nodes>
    	<Links>
    		<!-- Write links for activities nodes -->
	    	<xsl:for-each select="child::*">
				<xsl:variable name="allSourceLinks" select=".//bpel:source[@linkName=$allLinks/bpel:link/@name]"/>
				<xsl:for-each select="$allSourceLinks">
					<xsl:variable name="srcKeyName" select="concat('source',./@linkName)"/>
					<xsl:variable name="sourceNode" select="java:get($nodeSetVar,$srcKeyName)"/>
					<xsl:variable name="tarKeyName" select="concat('target',./@linkName)"/>
					<xsl:variable name="targetNode" select="java:get($nodeSetVar,$tarKeyName)"/>
	   				<xsl:call-template name="createLink">
   						<xsl:with-param name="source" select="xalan:nodeset($sourceNode)[position()=1]//ModelActivityElement/@UniqID"/>
   						<xsl:with-param name="target" select="xalan:nodeset($targetNode)[position()=1]//ModelActivityElement/@UniqID"/>
						<xsl:with-param name="targetType" select="name(xalan:nodeset($targetNode)/*[position()=1])"/>   					
   					</xsl:call-template>
				</xsl:for-each>
	    	</xsl:for-each>
    		<!-- Write the pre-created links -->
    		<!-- call getNodeSetString instead of get is to get around a strange problem that
    			sometimes the Link nodeset is corrupted when passed back from java to XSLT.
    		-->
	    	<xsl:value-of disable-output-escaping='yes' select="xalan:nodeset(java:getNodeSetString($nodeSetVar,$linksKey))"/> 	    	   
    	</Links>
	</xsl:template>
	
	<!-- 
	***************************************************************************************************************************************************************************
	Process Sequence activity.  This requires special treatment because there is no links for BPEL sequence 
	so we have to trace each nodes inside the sequence and re-generate the links
	***************************************************************************************************************************************************************************
-->
    <xsl:template name="processSequence">
    	<xsl:param name="currentPath"/>
   	    <xsl:variable name="prevElmKey" select="'PrevElement'"/>
   	    <xsl:variable name="srvNodeKey" select="'SrvNode'"/>
   		<xsl:variable name="resetLinks" select="java:reset($nodeSetVar,$linksKey)"/>		    			
    	<Nodes>
    	<xsl:for-each select="child::*">
    		<!-- check if it is the first element -->
    		<xsl:if test="not(boolean(preceding-sibling::*[position()=1]))">
	    		<xsl:variable name="firstPrevElement">
	    			<xsl:call-template name="createElement">
	    				<xsl:with-param name="elem" select="."/>
	    				<xsl:with-param name="currentPath" select="$currentPath"/>
	    			</xsl:call-template>
	    		</xsl:variable>
	    		<!-- fix the node if it is a service node -->
	    		<xsl:variable name="tempPrevNode">
	   				<xsl:call-template name="fixActionNode">
	   					<xsl:with-param name="inputNode" select="$firstPrevElement"/>
	   				</xsl:call-template>			    
				</xsl:variable>	    		
	    		<xsl:variable name="setPrevElm" select="java:set($nodeSetVar,$prevElmKey,xalan:nodeset($tempPrevNode))"/>
	    		<!-- Creates an initial node and link it with the currently created node
	    		 -->
		    	<xsl:variable name="initialNode">
		   			<xsl:call-template name="writeInitialNode"/>
				</xsl:variable>
    			<xsl:copy-of select="$initialNode"/>
    			<!-- Create Link between dummy initial node and first node -->
    			<xsl:variable name="source" select="xalan:nodeset($initialNode)[position()=1]//ModelActivityElement/@UniqID"/>
    			<xsl:variable name="target" select="xalan:nodeset($tempPrevNode)[position()=1]//ModelActivityElement/@UniqID"/>
    			<xsl:variable name="Link">
    				<xsl:call-template name="createLink">
    					<xsl:with-param name="source" select="$source"/>
    					<xsl:with-param name="target" select="$target"/>
						<xsl:with-param name="targetType" select="name(xalan:nodeset($tempPrevNode)/*[position()=1])"/>					
    				</xsl:call-template>
    			</xsl:variable>
	    		<xsl:variable name="newLinks" select="java:appendNodeAsString($nodeSetVar,$linksKey,$Link)"/>		    			
	    	</xsl:if>
    		<!-- 
    			To work around the problem with prevElement returned from java which for some reasons it
    			can't be interpreted as a nodeset
    		-->
	    	<xsl:variable name="tempPrevElement" select="java:get($nodeSetVar,$prevElmKey)"/>
    		<xsl:variable name="prevElement">
	    		<xsl:copy-of select="$tempPrevElement"/>
	    	</xsl:variable>	    	
   			<xsl:variable name="nodeType" select="name(following-sibling::*[position()=1])"/>
   			<xsl:variable name="isServiceNode" select="$nodeType='reply'or $nodeType='invoke' or $nodeType='receive'"/>
    		<xsl:variable name="nextElement">
			    <xsl:variable name="tempNext">
	    			<xsl:call-template name="createElement">
	    				<xsl:with-param name="elem" select="following-sibling::*[position()=1]"/>
	    				<xsl:with-param name="currentPath" select="$currentPath"/>
	    			</xsl:call-template>
	    		</xsl:variable>
   				<xsl:call-template name="fixActionNode">
   					<xsl:with-param name="inputNode" select="$tempNext"/>
   				</xsl:call-template>			    
    		</xsl:variable>
    		<!-- writes the previous element node -->
    		<xsl:copy-of select="$prevElement"/>
    		<!-- If the prevElement is a SubDiagram or Iterator node, then we need to keep track of the 
    			 name of the subdiagram reference so that when later on the diagram node is being created 
    			 for the subdiagram, we have to make sure the same name is used in the Diagram node and the 
    			 DiagramRef in the SubDiagram node.  This is useful if the name is being generated
    			 in the case that there is no name specified in 'sequence' or 'flow' tab which represents
    			 a subdiagram.
    		-->
    		<xsl:variable name="retPrevElem" select="$tempPrevElement/*[position()=1]"/>
    		<xsl:if test="name($retPrevElem)='SubDiagramNode' or name($retPrevElem)='IteratorNode'">
    			<xsl:variable name="subDiagRef" select="$retPrevElem/CompositeNode/@DiagramRef"/>
    			<xsl:variable name="subDiagRefListVar" select="java:get($simpleStrVar,$currentPath)"/>
    			<xsl:variable name="setSubDiagRefList" select="java:set($simpleStrVar,$currentPath,concat($subDiagRefListVar,$subDiagRef,';'))"/>
    		</xsl:if>
			<!-- Create Link Node if the currently processing node is not the last node -->
    		<xsl:if test="boolean(following-sibling::*[position()=1])">
    			<xsl:variable name="source" select="xalan:nodeset($prevElement)[position()=1]//ModelActivityElement/@UniqID"/>
    			<xsl:variable name="target" select="xalan:nodeset($nextElement)[position()=1]//ModelActivityElement/@UniqID"/>
    			<xsl:variable name="Link">
    				<xsl:call-template name="createLink">
    					<xsl:with-param name="source" select="$source"/>
    					<xsl:with-param name="target" select="$target"/>
						<xsl:with-param name="targetType" select="name(xalan:nodeset($nextElement)/*[position()=1])"/>					
    				</xsl:call-template>
    			</xsl:variable>
	    		<xsl:variable name="newLinks" select="java:appendNodeAsString($nodeSetVar,$linksKey,$Link)"/>
	    		<xsl:variable name="setPrevElm" select="java:set($nodeSetVar,$prevElmKey,xalan:nodeset($nextElement))"/>	    		
    		</xsl:if>
    	</xsl:for-each>
    	<!-- retrieve last processed node -->
    	<xsl:variable name="tempPrevElement" select="java:get($nodeSetVar,$prevElmKey)"/>
   		<xsl:variable name="prevElement">
    		<xsl:copy-of select="$tempPrevElement"/>
    	</xsl:variable>	    	
    	<!-- Create Success End Node -->
    	<xsl:if test="name(xalan:nodeset($prevElement)[position()=1]) != 'SuccessEndNode' and name(xalan:nodeset($prevElement)[position()=1]) != 'FailureEndNode'">
	    	<xsl:variable name="successEndNode"> 
				<xsl:call-template name="writeSuccessEndNode"/>
			</xsl:variable>
	    	<!-- Create Success End Node Link -->
	    	<xsl:variable name="endNodeLink">
	   			<xsl:variable name="source" select="xalan:nodeset($prevElement)[position()=1]//ModelActivityElement/@UniqID"/>
	   			<xsl:variable name="target" select="xalan:nodeset($successEndNode)[position()=1]//ModelActivityElement/@UniqID"/>
	   			<xsl:variable name="nodeLink">
	   				<xsl:call-template name="createLink">
	   					<xsl:with-param name="source" select="$source"/>
	   					<xsl:with-param name="target" select="$target"/>
						<xsl:with-param name="targetType" select="name(xalan:nodeset($successEndNode)/*[position()=1])"/>   					
	   				</xsl:call-template>
	   			</xsl:variable> 
		   		<xsl:variable name="appendLinks" select="java:appendNodeAsString($nodeSetVar,$linksKey,$nodeLink)"/>
		    </xsl:variable>
			<xsl:copy-of select="$successEndNode"/>
		</xsl:if>
    	</Nodes>
    	<Links>
    		<!-- call getNodeSetString instead of get is to get around a strange problem that
    			sometimes the Link nodeset is corrupted when passed back from java to XSLT
    		-->
	    	<xsl:value-of disable-output-escaping='yes' select="xalan:nodeset(java:getNodeSetString($nodeSetVar,$linksKey))"/> 	    	   
    	</Links>
    </xsl:template>
    
	<!-- 
	***************************************************************************************************************************************************************************
	Creates a node
	***************************************************************************************************************************************************************************
-->
    <xsl:template name="createElement">
    	<xsl:param name="elem"/>
    	<xsl:param name="currentPath"/>
    	<xsl:if test="boolean($elem)">
    		<xsl:apply-templates select="$elem[position()=1]" mode="Node">
    			<xsl:with-param name="currentPath" select="$currentPath"/>
    		</xsl:apply-templates>
    	</xsl:if>
    </xsl:template>

	<!-- 
	***************************************************************************************************************************************************************************
	Creates a link
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="createLink">
		<xsl:param name="source"/>
		<xsl:param name="target"/>
		<xsl:param name="targetType"/>
		<xsl:choose>
			<xsl:when test="$targetType='FailureEndNode'">
				<xsl:call-template name="writeExceptionLink">
					<xsl:with-param name="source" select="$source"/>
					<xsl:with-param name="target" select="$target"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="contains($targetType,'ServiceNode')">
				<xsl:call-template name="writeMessageLink">
					<xsl:with-param name="source" select="$source"/>
					<xsl:with-param name="target" select="$target"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name="writeTransitionLink">
					<xsl:with-param name="source" select="$source"/>
					<xsl:with-param name="target" select="$target"/>
				</xsl:call-template>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>	

	<!-- 
	***************************************************************************************************************************************************************************
	Fills in a Diagram Nodes
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="writeNodes">
		<xsl:variable name="nodes">
			<xsl:element name="Nodes">
				<xsl:apply-templates select="bpel:receive" mode="Node"/>
				<xsl:apply-templates select="bpel:assign" mode="Node"/>
				<xsl:apply-templates select="bpel:invoke" mode="Node"/>
				<xsl:apply-templates select="bpel:while" mode="Node"/>
				<xsl:apply-templates select="bpel:flow" mode="Node"/>
				<xsl:apply-templates select="bpel:empty" mode="Node"/>
				<xsl:apply-templates select="bpel:throw" mode="Node"/>
				<xsl:apply-templates select="bpel:sequence" mode="Node"/>
				<xsl:apply-templates select="wswf:script" mode="Node"/>
			</xsl:element>
		</xsl:variable>
		<xsl:apply-templates select="xalan:nodeset($nodes)//Nodes" mode="EliminateDuplicates"/>
	</xsl:template>
	
	<!-- 
	***************************************************************************************************************************************************************************
	EliminateDuplicates template is used for removing any nodes that has the same id and make sure there is only one node appear 
	in the nodes list with the same id.  This is a work around for InitialNode detection when there is
	mulitple links for the same InitialNode
	***************************************************************************************************************************************************************************
-->	
	<xsl:template match="Nodes" mode="EliminateDuplicates">
	    <xsl:variable name="uniq-nodes" select="*[not(*//ModelActivityElement/@UniqID = following::*//ModelActivityElement/@UniqID)]"/>
	    <Nodes>
		    <xsl:copy-of select="$uniq-nodes"/>
	    </Nodes>
	</xsl:template>
	
	<!-- 
	***************************************************************************************************************************************************************************
	Fills in the content of a sub-diagram or iterator node for sequence
	***************************************************************************************************************************************************************************
-->
	<xsl:template match="bpel:sequence" mode="Node">
		<xsl:param name="currentPath"/>
		<!-- Based on the CWT to BPEL convention, an iterator node is transformed to a while tag 
			 inside the sequence tag.  For BPEL to CWT, we need to handle this case plus we also
			 need to handle other cases where the sequence has other action nodes other than the
			 while.  So the rule here is if the sequence contains only a while, then we will treat
			 this sequence as an iterator node.  Otherwise we will treat the sequence as a sub-diagram
			 -->
		<xsl:choose>
			<xsl:when test="boolean(bpel:assign) or boolean(bpel:empty) or boolean(bpel:flow) or boolean(bpel:throw) or boolean(bpel:sequence) or boolean(bpel:invoke) or boolean(bpel:receive) or boolean(bpel:reply) or boolean(bpel:switch)">
				<xsl:call-template name="writeSubDiagramNode">
					<xsl:with-param name="currentPath" select="$currentPath"/>
				</xsl:call-template>			
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="bpel:while" mode="Node">
					<xsl:with-param name="currentPath" select="$currentPath"/>
				</xsl:apply-templates>			
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<!-- 
	***************************************************************************************************************************************************************************
	Fills in a Diagram for sequence
	***************************************************************************************************************************************************************************
-->
	
	<xsl:template match="bpel:sequence" mode="Diagram">
		<xsl:param name="currentPath"/>
		<xsl:choose>
			<xsl:when test="boolean(bpel:assign) or boolean(bpel:empty) or boolean(bpel:flow) or boolean(bpel:throw) or boolean(bpel:sequence) or boolean(bpel:invoke) or boolean(bpel:receive) or boolean(bpel:reply) or boolean(bpel:switch)">
				<xsl:variable name="newPath">
						<xsl:variable name="subDiagRefs" select="java:get($simpleStrVar,$currentPath)"/>
						<xsl:variable name="subDiagRef" select="substring-before($subDiagRefs,';')"/>
						<xsl:variable name="setSubDiagRef" select="java:set($simpleStrVar,$currentPath,substring-after($subDiagRefs,';'))"/>
						<xsl:choose>
							<xsl:when test="boolean(normalize-space($subDiagRef))">
								<xsl:value-of select="$subDiagRef"/>
							</xsl:when>
						</xsl:choose>	
				</xsl:variable>
				<xsl:call-template name="writeDiagram">
					<xsl:with-param name="name" select="$newPath"/>
					<xsl:with-param name="type" select="'sequence'"/>
					<xsl:with-param name="currentPath" select="$newPath"/>
				</xsl:call-template>
				<!-- need to call templates in sequence -->
				<xsl:for-each select="child::*">
					<xsl:variable name="nodeType" select="name(.)"/>
					<xsl:if test="$nodeType='sequence' or $nodeType='flow' or $nodeType='while'">
						<xsl:apply-templates select="." mode="Diagram">
							<xsl:with-param name="currentPath" select="$newPath"/>
						</xsl:apply-templates>
					</xsl:if>
				</xsl:for-each>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="bpel:while" mode="Diagram">
						<xsl:with-param name="currentPath" select="$currentPath"/>
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<!-- 
	***************************************************************************************************************************************************************************
	Fills in a Diagram Initial Node
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="detectInitialNode">
		<xsl:param name="targetLinkName"/>
		<xsl:apply-templates select="//bpel:pick/bpel:onMessage/bpel:sequence/bpel:empty[bpel:source/@linkName=$targetLinkName]" mode="Node"/>
	</xsl:template>
	<!-- 
	***************************************************************************************************************************************************************************
	Fills in a Node
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="writeNode">
		<xsl:param name="name"/>
		<xsl:param name="desc"/>
		<Node>
			<xsl:attribute name="UserModified">No</xsl:attribute>
			<xsl:call-template name="writeNodeModelActivityElement">
				<xsl:with-param name="name" select="$name"/>
				<xsl:with-param name="desc" select="$desc"/>
			</xsl:call-template>
		</Node>
	</xsl:template>
	<!-- 
	***************************************************************************************************************************************************************************
	Fills in a Diagram InitialNode
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="writeInitialNode">
		<xsl:param name="desc"/>
		<xsl:param name="name"/>
		<InitialNode>
			<PseudoState>
				<xsl:call-template name="writeNode">
					<xsl:with-param name="name" select="$name"/>
					<xsl:with-param name="desc" select="$desc"/>
				</xsl:call-template>
			</PseudoState>
		</InitialNode>
	</xsl:template>
	
	<!-- 
	***************************************************************************************************************************************************************************
	Fills in a Diagram SuccessEndNode
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="writeSuccessEndNode">
		<xsl:param name="desc"/>
		<xsl:param name="name"/>
		<SuccessEndNode>
			<PseudoState>
				<xsl:call-template name="writeNode">
					<xsl:with-param name="name" select="$name"/>
					<xsl:with-param name="desc" select="$desc"/>
				</xsl:call-template>
			</PseudoState>
		</SuccessEndNode>
	</xsl:template>
	
	<!-- 
	***************************************************************************************************************************************************************************
	Process an empty node
	***************************************************************************************************************************************************************************
-->
	<xsl:template match="bpel:empty" mode="Node">
		<xsl:choose>
			<xsl:when test="$CWBPEL">
				<xsl:choose>
					<!-- If the empty node has source but no target link then treat it as an initialNode.
						Othewise treat it as SuccessEndNode
					-->
					<xsl:when test="boolean(bpel:source) and not(boolean(bpel:target))">
						<xsl:call-template name="writeInitialNode">
							<xsl:with-param name="name" select="@name"/>
						</xsl:call-template>
					</xsl:when>
					<xsl:otherwise>
						<xsl:call-template name="writeSuccessEndNode">
							<xsl:with-param name="name" select="@name"/>
						</xsl:call-template>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:otherwise>
				<!-- for regular BPEL transformation, treat an empty node as an Action node -->
				<xsl:call-template name="writeActionNode">
					<xsl:with-param name="name" select="@name"/>
				</xsl:call-template>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- 
	***************************************************************************************************************************************************************************
	Process a switch node
	***************************************************************************************************************************************************************************
-->
	<xsl:template match="bpel:switch" mode="Node">
		<xsl:call-template name="writeActionNode">
			<xsl:with-param name="nodeName" select="@name"/>
		</xsl:call-template>
		<xsl:call-template name="detectInitialNode">
			<xsl:with-param name="targetLinkName" select="bpel:target/@linkName"/>
		</xsl:call-template>
	</xsl:template>

	<!-- 
	***************************************************************************************************************************************************************************
	Process an assign node
	***************************************************************************************************************************************************************************
-->
	<xsl:template match="bpel:assign" mode="Node">
		<xsl:call-template name="writeActionNode">
			<xsl:with-param name="nodeName" select="@name"/>
		</xsl:call-template>		
		<xsl:call-template name="detectInitialNode">
			<xsl:with-param name="targetLinkName" select="bpel:target/@linkName"/>
		</xsl:call-template>
	</xsl:template>
	
	<!-- 
	***************************************************************************************************************************************************************************
	Fills in a Diagram FailureEnd Nodes
	***************************************************************************************************************************************************************************
-->
	<xsl:template match="bpel:throw" mode="Node">
		<FailureEndNode>
			<PseudoState>
				<xsl:call-template name="writeNode">
					<xsl:with-param name="name">
						<xsl:value-of select="@name"/>
					</xsl:with-param>
				</xsl:call-template>
			</PseudoState>
		</FailureEndNode>
	</xsl:template>
	
	<!-- 
	***************************************************************************************************************************************************************************
	Process an invoke node
	***************************************************************************************************************************************************************************
-->
	<xsl:template match="bpel:invoke" mode="Node">
		<xsl:choose>
			<xsl:when test="$CWBPEL">
				<xsl:call-template name="writeServiceNode">			
					<xsl:with-param name="direction" select="'out'"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name="writeServiceNode"/>	
			</xsl:otherwise>
		</xsl:choose>
		<xsl:call-template name="detectInitialNode">
			<xsl:with-param name="targetLinkName" select="bpel:target/@linkName"/>
		</xsl:call-template>
	</xsl:template>
	
	<!-- 
	***************************************************************************************************************************************************************************
	Process a receive node
	***************************************************************************************************************************************************************************
-->
	<xsl:template match="bpel:receive" mode="Node">
		<xsl:choose>
			<xsl:when test="@createInstance='yes'">
				<xsl:call-template name="writeActionNode">
					<xsl:with-param name="desc" select="concat('partner=',@partner,' portType=',@portType,' operation=',@operation)"/>
					<xsl:with-param name="nodeName" select="@name"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name="writeServiceNode">
					<xsl:with-param name="direction" select="'in'"/>
				</xsl:call-template>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- 
	***************************************************************************************************************************************************************************
	Fills in a Diagram Service Node content
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="writeServiceNodeContent">
		<ServiceNode>
			<ProcessStep>
				<xsl:call-template name="writeNode">
					<xsl:with-param name="name">
						<xsl:value-of select="@name"/>
					</xsl:with-param>
				</xsl:call-template>
			</ProcessStep>
			<RegBusObjName>
				<xsl:choose>
					<xsl:when test="name(.)='invoke'">
						<xsl:value-of select="@inputContainer"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="@container"/>
					</xsl:otherwise>
				</xsl:choose>
			</RegBusObjName>
			<RegBusObjVerb>
				<xsl:value-of select="@operation"/>
			</RegBusObjVerb>
			<RegBusObjRefName>
				<xsl:choose>
					<xsl:when test="$CWBPEL">
						<xsl:value-of select="substring-before(substring-after(@portType,'pt:'),'PT')"/>
					</xsl:when>
					<xsl:otherwise>
					<!--
						<xsl:choose>
							<xsl:when test="name(.)='invoke'">
								<xsl:value-of select="@inputContainer"/>
							</xsl:when>
							<xsl:otherwise>
								<xsl:value-of select="@container"/>
							</xsl:otherwise>
						</xsl:choose>
						-->
						<!--
						<xsl:variable name="containerName">
							<xsl:choose>
								<xsl:when test="name(.)='invoke'">
									<xsl:value-of select="@inputContainer"/>
								</xsl:when>
								<xsl:otherwise>
									<xsl:value-of select="@container"/>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:variable>
						<xsl:variable name="containerType">
							<xsl:call-template name="getBusObjType">
								<xsl:with-param name="msgType" select="/bpel:process/bpel:containers/bpel:container[@name=$containerName]/@messageType"/>
							</xsl:call-template>
						</xsl:variable>
						-->
						<xsl:variable name="portName">
							<xsl:call-template name="getUniqueName">
								<xsl:with-param name="name" select="@name"/>
							</xsl:call-template>
						</xsl:variable>
						<!--
						<xsl:value-of select="concat($containerType,'_',$portName,'Port')"/>
						-->
						<xsl:value-of select="concat($portName,'Port')"/>
					</xsl:otherwise>
				</xsl:choose>
			</RegBusObjRefName>
			<CmpBusObjName>
				<xsl:choose>
					<xsl:when test="name(.)='invoke'">
						<xsl:value-of select="bpel:compensationHandler/bpel:invoke/@inputContainer"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="bpel:compensationHandler/bpel:invoke/@container"/>
					</xsl:otherwise>
				</xsl:choose>
			</CmpBusObjName>
			<CmpBusObjVerb>
				<xsl:value-of select="bpel:compensationHandler/bpel:invoke/@operation"/>
			</CmpBusObjVerb>
			<CmpBusObjRefName>
				<xsl:choose>
					<xsl:when test="$CWBPEL">
						<xsl:value-of select="substring-before(substring-after(bpel:compensationHandler/bpel:invoke/@portType,'pt:'),'PT')"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="substring-after(bpel:compensationHandler/bpel:invoke/@portType,'wPT:')"/>
					</xsl:otherwise>
				</xsl:choose>				
			</CmpBusObjRefName>
			<WaitTimeOut Type="NotUsed"/>			
		</ServiceNode>
	</xsl:template>
	
	<!-- 
	***************************************************************************************************************************************************************************
	Fills in a Diagram Service Node
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="writeServiceNode">
		<!-- direction: whether it is for in-bound or out-bound or bi-direction -->
		<xsl:param name="direction"/>
		<xsl:choose>
			<xsl:when test="$direction='in'">
				<InboundServiceNode>
					<xsl:attribute name="IsSync">true</xsl:attribute>
					<xsl:call-template name="writeServiceNodeContent"/>
				</InboundServiceNode>
			</xsl:when>
			<xsl:when test="$direction='out'">
				<OutboundServiceNode>
					<xsl:attribute name="IsSync">true</xsl:attribute>
					<xsl:call-template name="writeServiceNodeContent"/>
				</OutboundServiceNode>
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name="writeServiceNodeContent"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<!-- 
	***************************************************************************************************************************************************************************
	Process a reply node
	***************************************************************************************************************************************************************************
-->
	<xsl:template match="bpel:reply" mode="Node">
		<xsl:call-template name="writeServiceNode">
			<xsl:with-param name="direction" select="'out'"/>
		</xsl:call-template>
		<xsl:call-template name="detectInitialNode">
			<xsl:with-param name="targetLinkName" select="bpel:target/@linkName"/>
		</xsl:call-template>
	</xsl:template>
	
	<!-- 
	***************************************************************************************************************************************************************************
	Fills in a Diagram Iterator Node
	***************************************************************************************************************************************************************************
-->
	<xsl:template match="bpel:while" mode="Node">
		<xsl:param name="currentPath"/>
		<xsl:variable name="name">
			<xsl:call-template name="getDiagramName">
				<xsl:with-param name="currentPath" select="$currentPath"/>
				<xsl:with-param name="name" select="bpel:flow/@name"/>
				<xsl:with-param name="diagramType" select="'Iterator'"/>
			</xsl:call-template>		
		</xsl:variable>		

		<xsl:variable name="nodePreceding" select="./preceding-sibling::*" />
		<xsl:variable name="nodeNext" select="./child::wswf:script" />
		<xsl:variable name="bIsLoopType" select="not(normalize-space($nodePreceding[2]) = 'iteratorcounter = 0;')" />

		<!-- interpret the condition statement of the while action -->
		<xsl:variable name="strBusObjVariableFormat" select="substring-before(@condition,'(')"/>
		<xsl:variable name="strBusObj" select="normalize-space(substring-before($strBusObjVariableFormat,'.'))"/>
		<xsl:variable name="strBusObjChild" select="normalize-space(substring-after($strBusObjVariableFormat,'.get'))"/>


		<!-- interpret the condition statement of the while action -->
		<xsl:variable name="t1" select="substring-after(@condition,'(')"/>
		<xsl:variable name="t1x" select="substring-before(@condition,'(')"/>
		<xsl:variable name="t2">
			<xsl:value-of select="substring-before($t1,')')"/>
		</xsl:variable>
		
		<xsl:variable name="type">
			<xsl:choose>
				<xsl:when test="$strBusObjChild = '' and not($bIsLoopType)">Attribute</xsl:when>
				<xsl:when test="$bIsLoopType">Loop</xsl:when>
				<xsl:otherwise>BusObj</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<IteratorNode>
			<!-- New in Sapphire release -->
			<xsl:attribute name="Type">
				<xsl:value-of select="$type"/>
			</xsl:attribute>
			<CompositeNode>
				<xsl:attribute name="DiagramRef">
					<xsl:value-of select="$name"/>
				</xsl:attribute>
				<ProcessStep>
					<xsl:call-template name="writeNode">
						<xsl:with-param name="name" select="$name"/>
					</xsl:call-template>
				</ProcessStep>
			</CompositeNode>
			<IteratorVariable>
				<xsl:value-of select="$nodePreceding[1]"/>
			</IteratorVariable>
			<!-- BusinessObj -->
			<ThruThisObject>
				<xsl:choose>
					<xsl:when test="$CWBPEL">
						<xsl:choose>
							<xsl:when test="$strBusObjChild = '' and not($bIsLoopType)">
								<xsl:value-of select="$strBusObj" />
							</xsl:when>
							<xsl:otherwise>
								<xsl:value-of select="concat($strBusObj, '.', $strBusObjChild)" />								
							</xsl:otherwise>
						</xsl:choose>
					</xsl:when>
					<xsl:otherwise>
						<!-- for regular BPEL, assume it uses bpws:getContainerData function for condition -->
						<xsl:variable name="container" select="substring-before($t2,',')"/>
						<xsl:variable name="field" select="substring-after($t2,',')"/>
						<xsl:value-of select="concat(
								substring-before(substring-after($container,&quot;'&quot;),&quot;'&quot;),'.',
								substring-before(substring-after($field,&quot;'&quot;),&quot;'&quot;)
								)"/>
					</xsl:otherwise>
				</xsl:choose>
			</ThruThisObject>
			<xsl:choose>
				<xsl:when test="not($bIsLoopType)">
					<Start/>
					<Condition/>
					<Increment/>
				</xsl:when>
				<xsl:otherwise>
					<Start>
						<xsl:value-of select="$nodePreceding[2]"/>						
					</Start>
					<Condition>
						<xsl:value-of select="@condition"/>						
					</Condition>
					<Increment>	
						<xsl:value-of select="$nodeNext[1]"/>															
					</Increment>
				</xsl:otherwise>
			</xsl:choose>
		</IteratorNode>
		<xsl:call-template name="detectInitialNode">
			<xsl:with-param name="targetLinkName" select="bpel:target/@linkName"/>
		</xsl:call-template>
	</xsl:template>

	<!-- 
	***************************************************************************************************************************************************************************
	Fills in a Diagram Action Node
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="writeActionNode">
		<xsl:param name="codeFragment"/>
		<xsl:param name="desc"/>
		<xsl:param name="nodeName"/>
		<ActionNode>
			<ProcessStep>
				<xsl:call-template name="writeNode">
					<xsl:with-param name="name">
						<xsl:value-of select="$nodeName"/>
					</xsl:with-param>
					<xsl:with-param name="desc" select="$desc"/>
				</xsl:call-template>
			</ProcessStep>
			<CodeFragment Type="Java">
				<xsl:value-of select="$codeFragment"/>
			</CodeFragment>
		</ActionNode>		
	</xsl:template>
	
	<!-- 
	***************************************************************************************************************************************************************************
	Process a wswf:script node
	***************************************************************************************************************************************************************************
-->
	<xsl:template match="wswf:script" mode="Node">
		<xsl:variable name="name-text" select="@name"/>
		<xsl:choose>
			<xsl:when test="boolean(normalize-space($name-text))">
				<xsl:call-template name="writeActionNode">
					<xsl:with-param name="codeFragment" select="."/>
					<xsl:with-param name="nodeName" select="@name"/>
				</xsl:call-template>
			</xsl:when>
		</xsl:choose>
		<xsl:call-template name="detectInitialNode">
			<xsl:with-param name="targetLinkName" select="bpel:target/@linkName"/>
		</xsl:call-template>
	</xsl:template>
	
	<!-- 
	***************************************************************************************************************************************************************************
	Process a flow node
	***************************************************************************************************************************************************************************
-->
	<xsl:template match="bpel:flow" mode="Node">
		<xsl:param name="currentPath"/>
		<xsl:call-template name="writeSubDiagramNode">
			<xsl:with-param name="currentPath" select="$currentPath"/>
		</xsl:call-template>
	</xsl:template>
	
	<!-- 
	***************************************************************************************************************************************************************************
	Obtain the name of the diagram
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="getDiagramName">
		<xsl:param name="currentPath"/>
		<xsl:param name="name"/>		
		<xsl:param name="diagramType"/>
		<xsl:choose>
			<xsl:when test="boolean(normalize-space($name)) and $CWBPEL">
				<xsl:value-of select="$name"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:variable name="uniqID">
					<xsl:call-template name="getUniqueID"/>
				</xsl:variable>
				<xsl:value-of select="concat($currentPath,'.',$diagramType,$uniqID)"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<!-- 
	***************************************************************************************************************************************************************************
	Fills in a Diagram SubDiagram Node
	***************************************************************************************************************************************************************************
-->
	<xsl:template name="writeSubDiagramNode">
		<xsl:param name="currentPath"/>
		<xsl:variable name="diagName">
			<xsl:call-template name="getDiagramName">
				<xsl:with-param name="currentPath" select="$currentPath"/>
				<xsl:with-param name="name" select="@name"/>
				<xsl:with-param name="diagramType" select="'SubDiagram'"/>
			</xsl:call-template>		
		</xsl:variable>
		<SubDiagramNode>
			<CompositeNode>
				<xsl:attribute name="DiagramRef">
					<xsl:value-of select="$diagName"/>
				</xsl:attribute>
				<ProcessStep>
					<xsl:call-template name="writeNode">
						<xsl:with-param name="name">
							<xsl:value-of select="$diagName"/>
						</xsl:with-param>
					</xsl:call-template>
				</ProcessStep>
			</CompositeNode>
		</SubDiagramNode>
	</xsl:template>
	<!-- 
	***************************************************************************************************************************************************************************
	***************************************************************************************************************************************************************************
-->
</xsl:stylesheet>
