<?xml version="1.0" encoding="UTF-8"?>

<!--
	* Licensed Materials - Property of IBM
	* 5724-C11, 5724-C13, 5724-D17
	* (C) Copyright IBM Corporation 1997, 2003. All Rights Reserved.
	* US Government Users Restricted Rights- Use, duplication or disclosure
	* restricted by GSA ADP Schedule Contract with IBM Corp.
-->

<xsl:stylesheet 
	version="1.0" 
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:xalan="http://xml.apache.org/xalan"
	xmlns:java="http://xml.apache.org/xslt/java"	
	exclude-result-prefixes="xalan java" >

	<xsl:output method="xml" version="1.0" encoding="UTF-8" standalone="no" indent="yes"/>	

	<!-- 
	***************************************************************************************************************************************************************************
	Global variable definitions
	***************************************************************************************************************************************************************************
	-->

	<!-- *** Format utility variables *** -->
	<xsl:variable name="newline">
	<xsl:text>
	</xsl:text>
	</xsl:variable>	

	<xsl:variable name="objXSLTUtils" select="java:com.ibm.btools.itools.cwconverter.xslt.ext.XSLTUtils.new()"/>	

	<!-- 
	***************************************************************************************************************************************************************************
	Extract the information about the new decision node
	***************************************************************************************************************************************************************************
	-->

	<!-- *** XSLT starts at the root template *** -->
	<xsl:template match="/" >

		<xsl:variable name="nNewNodeID">
			<xsl:variable name="UniqIDsArray">
				<xsl:for-each select="CollabTemplate/Scenarios/Scenario[*]/ModelScenario/Diagram[*]/DiagramBody/Nodes[*]//ModelActivityElement |
				CollabTemplate/Scenarios/Scenario[*]/ModelScenario/Diagram[*]/DiagramBody/Links[*]//ModelActivityElement" >
				<xsl:sort select="substring-after(@UniqID, '_')" data-type="number" order="descending" /> 
					<xsl:copy-of select="." />
				</xsl:for-each>
			</xsl:variable>
			<xsl:variable name="NodeSetUniqIDs" select="xalan:nodeset($UniqIDsArray)" />

			<xsl:choose>
				<xsl:when test="count($NodeSetUniqIDs/*) &gt;= 1">
					<xsl:value-of select="java:setNextID($objXSLTUtils, substring-after($NodeSetUniqIDs/ModelActivityElement[1]/@UniqID, '_'))" />					
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="''" />					
				</xsl:otherwise>
			</xsl:choose>			
		</xsl:variable>			

		<xsl:choose>
			<xsl:when test="not(nNewNodeID = '')">
				<xsl:element name="Scenarios">
					<xsl:value-of select="$newline" />
					<xsl:for-each select="CollabTemplate/Scenarios/Scenario">
						<xsl:element name="Scenario">
							<xsl:element name="name">
								<xsl:value-of select="@Name" />
							</xsl:element>

							<xsl:for-each select="ModelScenario/Diagram">
								<xsl:element name="Diagram">
									<xsl:element name="name">
										<xsl:value-of select="@Name" />
									</xsl:element>

									<xsl:call-template name="writeDecisionInfo" />
								</xsl:element>
							</xsl:for-each>
						</xsl:element>
					</xsl:for-each>
				</xsl:element>
			</xsl:when>
			<!-- *** otherwise what should i do, need to format an empty element struct as expected by the next XSLT *** -->
			<xsl:otherwise>
				<xsl:element name="Scenarios" />
			</xsl:otherwise>
		</xsl:choose>

	</xsl:template>



	<xsl:template name="writeDecisionInfo">
				
		<xsl:variable name="LinksArray">
			<xsl:for-each select="DiagramBody/Links/*">						
			<xsl:sort select="substring-after(Link/Connect/@From, '_')" data-type="number" order="ascending" /> 
				<xsl:if test="not(name() = 'MessageLink')">
					<xsl:copy-of select="." />			
				</xsl:if>
			</xsl:for-each>			
		</xsl:variable>	

		<xsl:variable name="Links" select="xalan:nodeset($LinksArray)/*" />

		<xsl:variable name="temp" select="java:setCurrentSourceNodeID($objXSLTUtils,  0)" />			

		<xsl:for-each select="$Links">			
			<xsl:choose>
				<xsl:when test="java:getCurrentSourceNodeID($objXSLTUtils) = normalize-space(.//Link/Connect/@From)">
					<!-- ignore this -->					
				</xsl:when>
				<xsl:otherwise>
 					<xsl:variable name="FromNode" select="java:setCurrentSourceNodeID($objXSLTUtils,  normalize-space(.//Link/Connect/@From))" />			
					<xsl:variable name="SameFromNodeLinksCount" select="count($Links//Link[Connect/@From = $FromNode])" />				

					<xsl:if test="$SameFromNodeLinksCount &gt; 1"> 
						<xsl:element name="DecisionLinkNode">
							<xsl:attribute name="FirstLinkPosition">
							<xsl:value-of select="position()" /> 
							</xsl:attribute>
							<xsl:attribute name="LastLinkPosition" > 
							<xsl:value-of select="position() + $SameFromNodeLinksCount - 1" /> 
							</xsl:attribute>
							<xsl:element name="NewDecisionNodeID" >
								<xsl:value-of select="concat('_', java:getNextID($objXSLTUtils))" />
							</xsl:element>
							<xsl:element name="NewLinkNodeID" >
								<xsl:value-of select="concat('_', java:getNextID($objXSLTUtils))" />
							</xsl:element>
							<xsl:element name="SourceNodeID" >
								<xsl:value-of select=".//Link/Connect/@From" />
							</xsl:element>
							<xsl:element name="Links" >
								<xsl:call-template name="recursiveGetLinkID" >
									<xsl:with-param name="LastLinkPosition" select="(position() + $SameFromNodeLinksCount - 1)" />
									<xsl:with-param name="CurrentLinkPosition" select="position()" />
									<xsl:with-param name="Links" select="$Links" />
								</xsl:call-template>
							</xsl:element>
						</xsl:element>
						<xsl:value-of select="$newline" />		
					</xsl:if>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:for-each>
	</xsl:template>
			

	<xsl:template name="recursiveGetLinkID">
		<xsl:param name="LastLinkPosition" />
		<xsl:param name="CurrentLinkPosition" />
		<xsl:param name="Links" />
		<xsl:element name="LinkInfo">
			<xsl:attribute name="ID" >
				<xsl:value-of select="$Links[$CurrentLinkPosition]//Link/ModelActivityElement/@UniqID" />
			</xsl:attribute>
			<xsl:attribute name="ToID" >
				<xsl:value-of select="$Links[$CurrentLinkPosition]//Link/Connect/@To" />
			</xsl:attribute>
			<xsl:choose>
				<xsl:when test="name($Links[$CurrentLinkPosition]) = 'ExceptionLink'">
					<xsl:element name="LinkDetails">
						<xsl:attribute name="type">
							<xsl:value-of select="'Exception'" />
						</xsl:attribute>
						<xsl:element name="Value">
							<xsl:value-of select="$Links[$CurrentLinkPosition]//ExceptionName" />
						</xsl:element>
						<xsl:element name="Comment">
							<xsl:value-of select="$Links[$CurrentLinkPosition]//Link/ModelActivityElement/ModelElement/Desc" />
						</xsl:element>
						<xsl:element name="Label">
							<xsl:value-of select="$Links[$CurrentLinkPosition]//Link/ModelActivityElement/Label" />
						</xsl:element>
					</xsl:element>
				</xsl:when>
				<xsl:when test="name($Links[$CurrentLinkPosition]) = 'TransitionLink'">
					<xsl:element name="LinkDetails">
						<xsl:attribute name="type">
							<xsl:value-of select="'Normal'" />
						</xsl:attribute>
						<xsl:element name="Value">
							<xsl:value-of select="$Links[$CurrentLinkPosition]//GuardCondition" />
						</xsl:element>
						<xsl:element name="Comment">
							<xsl:value-of select="$Links[$CurrentLinkPosition]//Link/ModelActivityElement/ModelElement/Desc" />
						</xsl:element>
						<xsl:element name="Label">
							<xsl:value-of select="$Links[$CurrentLinkPosition]//Link/ModelActivityElement/Label" />
						</xsl:element>
					</xsl:element>
				</xsl:when>
				<xsl:otherwise>
					<xsl:element name="LinkDetails">
						<xsl:attribute name="type">
							<xsl:value-of select="'Unknown'" />
						</xsl:attribute>
						<xsl:element name="Value">
							<xsl:value-of select="''" />
						</xsl:element>
					</xsl:element>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:element>


		<xsl:if test="(($CurrentLinkPosition + 1) &lt;=  $LastLinkPosition) and (($CurrentLinkPosition + 1) &lt;= count($Links))" >
				<xsl:call-template name="recursiveGetLinkID" >
					<xsl:with-param name="LastLinkPosition" select="($LastLinkPosition)" />
					<xsl:with-param name="CurrentLinkPosition" select="($CurrentLinkPosition + 1)" />
					<xsl:with-param name="Links" select="$Links" />
				</xsl:call-template>
		</xsl:if> 

	</xsl:template>		

</xsl:stylesheet>
