<?xml version="1.0" encoding="UTF-8"?>

<xsl:stylesheet 
	version="1.0" 
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 	
	xmlns:xalan="http://xml.apache.org/xalan"
	exclude-result-prefixes="xalan">

	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/>	

	<xsl:strip-space elements="*" />

	<!-- 
	***************************************************************************************************************************************************************************
	Global variable definitions
	***************************************************************************************************************************************************************************
	-->

	<!-- *** Format utility variables *** -->

	<xsl:variable name="newline">
	<xsl:text>
	</xsl:text>
	</xsl:variable>	

	<xsl:template match="/">
		<xsl:element name="Process">

			<xsl:if test="not(/CollabTemplate/TemplateDefinition/Properties/Property[@Name = 'SystemProperties']/Attributes/Attribute[@Name = 'LongLivedBusinessProcess']/Value = 'false') and count(/CollabTemplate/Scenarios/Scenario[*]/ModelScenario/Diagram[*]/DiagramBody/Nodes/*//CorrelationSet[@IsInit = 'Yes']/CorrelationAttribute[*]/VariableName) &gt; 0">
				<xsl:element name="CorrelationSets">
					<xsl:variable name="treeSets">					
						<xsl:for-each select="/CollabTemplate/Scenarios/Scenario[*]/ModelScenario/Diagram[*]/DiagramBody/Nodes/*//CorrelationSet[@IsInit = 'Yes']">
							<xsl:element name="CorrelationSet">
								<xsl:attribute name="Properties">
									<xsl:for-each select="xalan:distinct(CorrelationAttribute[*]/VariableName)">
										<xsl:sort select="." data-type="text" />
											<xsl:value-of select="concat(' ', 'wsdlWBI:', normalize-space(.))" />									
									</xsl:for-each>
								</xsl:attribute>
							</xsl:element>
						</xsl:for-each>				
					</xsl:variable>

					<xsl:variable name="nodeSets" select="xalan:nodeset($treeSets)" />

					<xsl:for-each select="xalan:distinct($nodeSets/CorrelationSet/@Properties)">
							<xsl:element name="CorrelationSet">
								<xsl:attribute name="pd.name">
									<xsl:value-of select="concat('correlationSet_', position())" />
								</xsl:attribute>
								<xsl:attribute name="pd.properties">
									<xsl:value-of select="." />									
								</xsl:attribute>
							</xsl:element>						
					</xsl:for-each>
				</xsl:element>
			</xsl:if>

			<xsl:element name="Scenarios">
				<xsl:for-each select="/CollabTemplate/Scenarios/Scenario">			
					<xsl:element name="Scenario">
						<xsl:attribute name="Name">
							<xsl:value-of select="@Name" />						
						</xsl:attribute>

						<xsl:variable name="strMainDiagramName" select="@DiagramRef" />
						<xsl:variable name="strGUIMainDiagramName" select="@DiagramRefGUI" />
											
						<xsl:if test="count(GUIDiagrams/GUIDiagram[@Name = $strGUIMainDiagramName]) = 1">					
							<xsl:for-each select="GUIDiagrams/GUIDiagram[@Name = $strGUIMainDiagramName]">												
								<xsl:call-template name="writeDiagramNodes">
									<xsl:with-param name="nodeDiagram" select="../../ModelScenario/Diagram[@Name = $strMainDiagramName]" />
								</xsl:call-template>
							</xsl:for-each>
						</xsl:if>				
					</xsl:element>
				</xsl:for-each>						
			</xsl:element>
		</xsl:element>
	</xsl:template>

	<xsl:template name="writeDiagramNodes">
	<xsl:param name="nodeDiagram" />
<!-- *** now as we are ignoring the decision node and converting the BO probe's extra nodes, we can just go through the 
		 ModelScenario/Diagram elements rather than the GUIDiagrams/GUIDiagram where we got real diagram information 
	 ***
-->
		<xsl:element name="pd.nodes">
			<xsl:for-each select="$nodeDiagram/DiagramBody/Nodes/*">
				<xsl:choose>
					<xsl:when test="name(.) = 'InitialNode'">
						<xsl:call-template name="writeInitialNode" />
					</xsl:when>
					<xsl:when test="name(.) = 'ActionNode'">
						<xsl:call-template name="writeActionNode" />
					</xsl:when>
					<xsl:when test="name(.) = 'SuccessEndNode'">
						<xsl:call-template name="writeSuccessEndNode" />
					</xsl:when>
					<xsl:when test="name(.) = 'FailureEndNode'">
						<xsl:call-template name="writeFailureEndNode" />
					</xsl:when>
					<xsl:when test="name(.) = 'OutboundServiceNode'">
						<xsl:call-template name="writeOutboundServiceNode" />
					</xsl:when>
					<xsl:when test="name(.) = 'InboundServiceNode'">
						<xsl:call-template name="writeInboundServiceNode" />
					</xsl:when>
					<xsl:when test="name(.) = 'IteratorNode'">
						<xsl:call-template name="writeIteratorNode" />
					</xsl:when>
					<xsl:when test="name(.) = 'SubDiagramNode'">
						<xsl:call-template name="writeSubDiagramNode" />
					</xsl:when>
					<xsl:when test="name(.) = 'BreakNode'">
						<xsl:call-template name="writeBreakNode" />
					</xsl:when>
				</xsl:choose>						
			</xsl:for-each>
		</xsl:element>
	</xsl:template>

	<xsl:template name="writeInitialNode">
		<xsl:element name="{name()}">
			<xsl:attribute name="pd.id">
				<xsl:value-of select=".//ModelActivityElement/@UniqID" />
			</xsl:attribute>
			<xsl:attribute name="pd.label">
				<xsl:value-of select="translate(.//ModelActivityElement/Label, '&#10;', ' ')" />						
			</xsl:attribute>
			<xsl:element name="pd.desc">
				<xsl:value-of select=".//ModelActivityElement/ModelElement/Desc" />						
			</xsl:element>

			<xsl:call-template name="writeCorrelationSetInfo">
				<xsl:with-param name="nodeCorrelationSet" select="CorrelationSet" />
			</xsl:call-template>			

			<xsl:call-template name="writeSourceandTargetLinks">
				<xsl:with-param name="strUniqueID" select=".//ModelActivityElement/@UniqID" />
			</xsl:call-template>

		</xsl:element>

	</xsl:template>

	<xsl:template name="writeActionNode">
		<xsl:element name="{name()}">
			<xsl:attribute name="pd.id">
				<xsl:value-of select=".//ModelActivityElement/@UniqID" />
			</xsl:attribute>
			<xsl:attribute name="pd.label">
				<xsl:value-of select="translate(.//ModelActivityElement/Label, '&#10;', ' ')" />						
			</xsl:attribute>
			<xsl:element name="pd.desc">
				<xsl:value-of select=".//ModelActivityElement/ModelElement/Desc" />
			</xsl:element>
			<xsl:element name="pd.codefragment">
				<xsl:attribute name="pd.type">  
					<xsl:value-of select="./CodeFragment/@Type" /> 
				</xsl:attribute> 							
				<xsl:value-of select="./CodeFragment" /> 								
			</xsl:element>

			<xsl:call-template name="writeSourceandTargetLinks">
				<xsl:with-param name="strUniqueID" select=".//ModelActivityElement/@UniqID" />
			</xsl:call-template>

		</xsl:element>

	</xsl:template>

	<xsl:template name="writeOutboundServiceNode">
		<xsl:element name="{name()}">
			<xsl:attribute name="pd.id">
				<xsl:value-of select=".//ModelActivityElement/@UniqID" />
			</xsl:attribute>
			<xsl:attribute name="pd.label">
				<xsl:value-of select="translate(.//ModelActivityElement/Label, '&#10;', ' ')" />						
			</xsl:attribute>
			<xsl:element name="pd.desc">
				<xsl:value-of select=".//ModelActivityElement/ModelElement/Desc" />
			</xsl:element>
			<xsl:element name="pd.service.isSync">
				<xsl:value-of select="@IsSync" />
			</xsl:element>

			<xsl:element name="pd.port">
				<xsl:value-of select="ServiceNode/RegBusObjRefName" />
			</xsl:element>
			<xsl:element name="pd.verb">
				<xsl:value-of select="ServiceNode/RegBusObjVerb" />
			</xsl:element>
			<xsl:element name="pd.busobjvariable">
				<xsl:value-of select="ServiceNode/RegBusObjName" />
			</xsl:element>

			<xsl:element name="pd.comp.port">
				<xsl:value-of select="ServiceNode/CmpBusObjRefName" />
			</xsl:element>
			<xsl:element name="pd.comp.verb">
				<xsl:value-of select="ServiceNode/CmpBusObjVerb" />
			</xsl:element>
			<xsl:element name="pd.comp.busobjvariable">
				<xsl:value-of select="ServiceNode/CmpBusObjName" />
			</xsl:element>

			<xsl:element name="pd.wait.timeout">
				<xsl:attribute name="pd.type">
					<xsl:value-of select="ServiceNode/WaitTimeOut/@Type" />						
				</xsl:attribute>				
				<xsl:value-of select="ServiceNode/WaitTimeOut" />
			</xsl:element>

			<xsl:call-template name="writeCorrelationSetInfo">
				<xsl:with-param name="nodeCorrelationSet" select="ServiceNode/CorrelationSet" />
			</xsl:call-template>			

			<xsl:call-template name="writeSourceandTargetLinksForServiceCall">
				<xsl:with-param name="strUniqueID" select=".//ModelActivityElement/@UniqID" />
			</xsl:call-template>

		</xsl:element>
	</xsl:template>

	<xsl:template name="writeInboundServiceNode">
		<xsl:element name="{name()}">
			<xsl:attribute name="pd.id">
				<xsl:value-of select=".//ModelActivityElement/@UniqID" />
			</xsl:attribute>
			<xsl:attribute name="pd.label">
				<xsl:value-of select="translate(.//ModelActivityElement/Label, '&#10;', ' ')" />						
			</xsl:attribute>
			<xsl:element name="pd.desc">
				<xsl:value-of select=".//ModelActivityElement/ModelElement/Desc" />
			</xsl:element>

			<xsl:element name="pd.port">
				<xsl:value-of select="ServiceNode/RegBusObjRefName" />
			</xsl:element>
			<xsl:element name="pd.verb">
				<xsl:value-of select="ServiceNode/RegBusObjVerb" />
			</xsl:element>
			<xsl:element name="pd.busobjvariable">
				<xsl:value-of select="ServiceNode/RegBusObjName" />
			</xsl:element>

			<xsl:element name="pd.wait.timeout">
				<xsl:attribute name="pd.type">
					<xsl:value-of select="ServiceNode/WaitTimeOut/@Type" />						
				</xsl:attribute>				
				<xsl:value-of select="ServiceNode/WaitTimeOut" />
			</xsl:element>

			<xsl:call-template name="writeCorrelationSetInfo">
				<xsl:with-param name="nodeCorrelationSet" select="//CorrelationSet" />
			</xsl:call-template>			

			<xsl:call-template name="writeSourceandTargetLinksForServiceCall">
				<xsl:with-param name="strUniqueID" select=".//ModelActivityElement/@UniqID" />
			</xsl:call-template>

		</xsl:element>
	</xsl:template>

	<xsl:template name="writeIteratorNode">
		<xsl:element name="{name()}">
			<xsl:attribute name="pd.id">
				<xsl:value-of select=".//ModelActivityElement/@UniqID" />
			</xsl:attribute>
			<xsl:attribute name="pd.label">
				<xsl:value-of select="translate(.//ModelActivityElement/Label, '&#10;', ' ')" />						
			</xsl:attribute>
			<xsl:element name="pd.desc">
				<xsl:value-of select=".//ModelActivityElement/ModelElement/Desc" />
			</xsl:element>
			<xsl:element name="pd.type">
				<xsl:value-of select="@Type" />
			</xsl:element>
			<xsl:element name="pd.temp.variable">
				<xsl:value-of select="IteratorVariable" />
			</xsl:element>
			<xsl:element name="pd.object">
				<xsl:value-of select="ThruThisObject" />
			</xsl:element>
			<xsl:element name="pd.start">
				<xsl:value-of select="Start" />
			</xsl:element>
			<xsl:element name="pd.condition">
				<xsl:value-of select="Condition" />
			</xsl:element>
			<xsl:element name="pd.increment">
				<xsl:value-of select="Increment" />
			</xsl:element>
			

			<xsl:call-template name="writeSourceandTargetLinks">
				<xsl:with-param name="strUniqueID" select=".//ModelActivityElement/@UniqID" />
			</xsl:call-template>

			<!-- *** get the subdiagram info *** -->			
			<xsl:variable name="subDiagramName" select="concat('GUI__', CompositeNode/@DiagramRef)" />

			<xsl:if test="count(../../../../../GUIDiagrams/GUIDiagram[@Name = $subDiagramName]) = 1">
				<xsl:for-each select="../../../../../GUIDiagrams/GUIDiagram[@Name = $subDiagramName]">
					<xsl:variable name="strDiagramName" select="substring-after(@Name, 'GUI__')" />
					<xsl:call-template name="writeDiagramNodes">
						<xsl:with-param name="nodeDiagram" select="../../ModelScenario/Diagram[@Name = $strDiagramName]" />
					</xsl:call-template>
				</xsl:for-each>
			</xsl:if>							
		</xsl:element>
	</xsl:template>

	<xsl:template name="writeSubDiagramNode">
		<xsl:element name="{name()}">
			<xsl:attribute name="pd.id">
				<xsl:value-of select=".//ModelActivityElement/@UniqID" />
			</xsl:attribute>
			<xsl:attribute name="pd.label">
				<xsl:value-of select="translate(.//ModelActivityElement/Label, '&#10;', ' ')" />						
			</xsl:attribute>
			<xsl:element name="pd.desc">
				<xsl:value-of select=".//ModelActivityElement/ModelElement/Desc" />
			</xsl:element>


			<xsl:call-template name="writeSourceandTargetLinks">
				<xsl:with-param name="strUniqueID" select=".//ModelActivityElement/@UniqID" />
			</xsl:call-template>

			<!-- *** get the subdiagram info *** -->
			<xsl:variable name="subDiagramName" select="concat('GUI__', CompositeNode/@DiagramRef)" />			

			<xsl:if test="count(../../../../../GUIDiagrams/GUIDiagram[@Name = $subDiagramName]) = 1">
				<xsl:for-each select="../../../../../GUIDiagrams/GUIDiagram[@Name = $subDiagramName]">					
					<xsl:variable name="strDiagramName" select="substring-after(@Name, 'GUI__')" />						
					<xsl:call-template name="writeDiagramNodes">
						<xsl:with-param name="nodeDiagram" select="../../ModelScenario/Diagram[@Name = $strDiagramName]" />
					</xsl:call-template>
				</xsl:for-each>
			</xsl:if>				
		</xsl:element>
	</xsl:template>

	<xsl:template name="writeSuccessEndNode">
		<xsl:element name="{name()}">
			<xsl:attribute name="pd.id">
				<xsl:value-of select=".//ModelActivityElement/@UniqID" />
			</xsl:attribute>
			<xsl:attribute name="pd.label">
				<xsl:value-of select="translate(.//ModelActivityElement/Label, '&#10;', ' ')" />						
			</xsl:attribute>
			<xsl:element name="pd.desc">
				<xsl:value-of select=".//ModelActivityElement/ModelElement/Desc" />
			</xsl:element>

			<xsl:call-template name="writeSourceandTargetLinks">
				<xsl:with-param name="strUniqueID" select=".//ModelActivityElement/@UniqID" />
			</xsl:call-template>

		</xsl:element>
	</xsl:template>

	<xsl:template name="writeFailureEndNode">
		<xsl:element name="{name()}">
			<xsl:attribute name="pd.id">
				<xsl:value-of select=".//ModelActivityElement/@UniqID" />
			</xsl:attribute>
			<xsl:attribute name="pd.label">
				<xsl:value-of select="translate(.//ModelActivityElement/Label, '&#10;', ' ')" />						
			</xsl:attribute>
			<xsl:element name="pd.desc">
				<xsl:value-of select=".//ModelActivityElement/ModelElement/Desc" />
			</xsl:element>

			<xsl:call-template name="writeSourceandTargetLinks">
				<xsl:with-param name="strUniqueID" select=".//ModelActivityElement/@UniqID" />
			</xsl:call-template>

		</xsl:element>
	</xsl:template>

	<xsl:template name="writeBreakNode">
		<xsl:element name="{name()}">
			<xsl:attribute name="pd.id">
				<xsl:value-of select=".//ModelActivityElement/@UniqID" />
			</xsl:attribute>
			<xsl:attribute name="pd.label">
				<xsl:value-of select="translate(.//ModelActivityElement/Label, '&#10;', ' ')" />						
			</xsl:attribute>
			<xsl:element name="pd.desc">
				<xsl:value-of select=".//ModelActivityElement/ModelElement/Desc" />
			</xsl:element>

			<xsl:call-template name="writeSourceandTargetLinks">
				<xsl:with-param name="strUniqueID" select=".//ModelActivityElement/@UniqID" />
			</xsl:call-template>

		</xsl:element>
	</xsl:template>

	<xsl:template name="writeCorrelationSetInfo">
	<xsl:param name="nodeCorrelationSet" />
		<xsl:element name="CorrelationSet">
			<xsl:attribute name="IsInit">
				<xsl:value-of select="$nodeCorrelationSet/@IsInit" />
			</xsl:attribute>
			<xsl:attribute name="pd.correlation.properties">
				<xsl:for-each select="xalan:distinct($nodeCorrelationSet/CorrelationAttribute[*]/VariableName)">
					<xsl:sort select="." data-type="text" />
						<xsl:value-of select="concat(' ', 'wsdlWBI:', normalize-space(.))" />									
				</xsl:for-each>				
			</xsl:attribute>
			<xsl:for-each select="$nodeCorrelationSet/CorrelationAttribute">
				<xsl:copy-of select="." />
			</xsl:for-each>
		</xsl:element>
		
	</xsl:template>			


	<xsl:template name="writeSourceandTargetLinks">
	<xsl:param name="strUniqueID" />
		<xsl:element name="pd.links">
			<xsl:element name="pd.sources">
				<xsl:for-each select="../../Links/*[Link/Connect/@From = $strUniqueID]">
					<xsl:element name="pd.link">
						<xsl:attribute name="pd.id">
							<xsl:value-of select=".//ModelActivityElement/@UniqID" />
						</xsl:attribute>
						<xsl:attribute name="pd.label">
							<xsl:value-of select="translate(.//ModelActivityElement/Label, '&#10;', ' ')" />						
						</xsl:attribute>
						<xsl:attribute name="type">
							<xsl:value-of select="name(.)" />
						</xsl:attribute>
						<xsl:element name="pd.desc">
							<xsl:value-of select=".//ModelActivityElement/ModelElement/Desc" />
						</xsl:element>

						<xsl:element name="pd.condition">							
								<xsl:choose>
									<xsl:when test="name(.) = 'ExceptionLink'">
										<xsl:attribute name="type">
											<xsl:value-of select="'Exception'" />
										</xsl:attribute>
										<xsl:value-of select="ExceptionName" />
									</xsl:when>
									<xsl:otherwise>
										<xsl:attribute name="type">
											<xsl:value-of select="'Normal'" />
										</xsl:attribute>
										<xsl:value-of select="GuardCondition" />
									</xsl:otherwise>
								</xsl:choose>							
						</xsl:element>						
					</xsl:element>
				</xsl:for-each>
			</xsl:element>
			<xsl:element name="pd.targets">
				<xsl:for-each select="../../Links/*[Link/Connect/@To = $strUniqueID]">
					<xsl:element name="pd.link">
						<xsl:attribute name="pd.id">
							<xsl:value-of select=".//ModelActivityElement/@UniqID" />
						</xsl:attribute>
						<xsl:attribute name="pd.label">
							<xsl:value-of select="translate(.//ModelActivityElement/Label, '&#10;', ' ')" />						
						</xsl:attribute>
						<xsl:attribute name="type">
							<xsl:value-of select="name(.)" />
						</xsl:attribute>
					</xsl:element>
				</xsl:for-each>
			</xsl:element>
		</xsl:element>
	</xsl:template>

	<xsl:template name="writeSourceandTargetLinksForServiceCall">
	<xsl:param name="strUniqueID" />
		<xsl:element name="pd.links">
			<xsl:element name="pd.sources">
				<xsl:variable name="strActionNodeID" select="../../Links/*[Link/Connect/@To = $strUniqueID]/Link/Connect/@From" />
				<xsl:for-each select="../../Links/*[Link/Connect/@From = $strActionNodeID]">
					<xsl:element name="pd.link">
						<xsl:attribute name="pd.id">
							<xsl:value-of select=".//ModelActivityElement/@UniqID" />
						</xsl:attribute>
						<xsl:attribute name="pd.label">
							<xsl:value-of select="translate(.//ModelActivityElement/Label, '&#10;', ' ')" />						
						</xsl:attribute>
						<xsl:attribute name="type">
							<xsl:value-of select="name(.)" />
						</xsl:attribute>
						<xsl:element name="pd.desc">
							<xsl:value-of select=".//ModelActivityElement/ModelElement/Desc" />
						</xsl:element>

						<xsl:element name="pd.condition">							
								<xsl:choose>
									<xsl:when test="name(.) = 'ExceptionLink'">
										<xsl:attribute name="type">
											<xsl:value-of select="'Exception'" />
										</xsl:attribute>
										<xsl:value-of select="ExceptionName" />
									</xsl:when>
									<xsl:otherwise>
										<xsl:attribute name="type">
											<xsl:value-of select="'Normal'" />
										</xsl:attribute>
										<xsl:value-of select="GuardCondition" />
									</xsl:otherwise>
								</xsl:choose>							
						</xsl:element>						
					</xsl:element>
				</xsl:for-each>
			</xsl:element>
			<xsl:element name="pd.targets">
				<xsl:for-each select="../../Links/*[Link/Connect/@To = $strUniqueID]">
					<xsl:element name="pd.link">
						<xsl:attribute name="pd.id">
							<xsl:value-of select=".//ModelActivityElement/@UniqID" />
						</xsl:attribute>
						<xsl:attribute name="pd.label">
							<xsl:value-of select="translate(.//ModelActivityElement/Label, '&#10;', ' ')" />						
						</xsl:attribute>
						<xsl:attribute name="type">
							<xsl:value-of select="name(.)" />
						</xsl:attribute>
					</xsl:element>
				</xsl:for-each>
			</xsl:element>
		</xsl:element>
	</xsl:template>

<!-- *** Design Document ***

	collect the informations about the nodes inside eac scenarion for latter use

		1. for each scenario, start collection information from the main scenario diagram and then go deep in to all sub diagrams used by sub nodes and iterator nodes.

	 *** Design Document ***
-->


</xsl:stylesheet>
